# 公共常量 >>>>>>>>>>>>>>>>>>>>>>>>>>>
GRPC_MAX_MESSAGE_LENGTH = 32*1024*1024

# 公共常量 <<<<<<<<<<<<<<<<<<<<<<<<<<

# GRPC 常量 >>>>>>>>>>>>>>>>>>>>>>>>>
CAN_STACK_NODE_IP = '127.0.0.1'
CAN_STACK_NODE_PORT = 6001
CAN_PARSERNODE_NODE_IP = '127.0.0.1'
CAN_PARSERNODE_NODE_PORT = 6005
LIN_STACK_NODE_IP = '127.0.0.1'
LIN_STACK_NODE_PORT = 6003
LIN_PARSERNODE_NODE_IP = '127.0.0.1'
LIN_PARSERNODE_NODE_PORT = 6004
SOMEIP_STACK_NODE_IP = '127.0.0.1'
SOMEIP_STACK_NODE_PORT = 6002
LOG_REPLAY_NODE_IP = '127.0.0.1'
LOG_REPLAY_NODE_PORT = 6007

PROCESS_NODE_IP = '127.0.0.1'
PROCESS_NODE_PORT = 6008

GRPC_OPTIONS = [
    ('grpc.max_send_message_length', GRPC_MAX_MESSAGE_LENGTH),
    ('grpc.max_receive_message_length', GRPC_MAX_MESSAGE_LENGTH),
]

# GRPC 常量 <<<<<<<<<<<<<<<<<<<<<<<<<<

# MQTT 常量 >>>>>>>>>>>>>>>>>>>>>>>>>
MQTT_BROKER_IP = '127.0.0.1'
MQTT_BROKER_PORT = 8083

# MQTT 常量 <<<<<<<<<<<<<<<<<<<<<<<<<


import typing
import copy
from .ZoneSenderObject import ZoneSenderObject


class LinSignal(ZoneSenderObject):
    ''' LIN Signal 对象，用来描述一个 LIN Signal
    '''
    def __init__(
        self,
        name: 'str',
        channel: 'int',
        data_raw: 'float' = 0.0,
        data = None,
        unit: 'str' = ''
        ) -> None:
        ...
        # #####################################
        super().__init__()
        self.name = name
        self.data_raw = data_raw
        self.channel = channel
        self.data = data
        self.unit = unit
        self.id = None

    def __str__(self) -> str:
        # return super().__str__()
        return 'name: {0} channel: {1} value: {2} '.format(self.name, self.channel, self.data)


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import Common_pb2 as Common__pb2
from . import LogReplayNode_pb2 as LogReplayNode__pb2


class LogReplayNodeStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartLog = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/StartLog',
                request_serializer=LogReplayNode__pb2.log_request.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.GetLogStatus = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/GetLogStatus',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.generic_string.FromString,
                )
        self.StopLog = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/StopLog',
                request_serializer=LogReplayNode__pb2.log_request.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.AddLinDecodeRole = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/AddLinDecodeRole',
                request_serializer=Common__pb2.generic_string.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.AddCanDecodeRole = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/AddCanDecodeRole',
                request_serializer=Common__pb2.generic_string.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.ReplayNextN = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/ReplayNextN',
                request_serializer=LogReplayNode__pb2.decode_n.SerializeToString,
                response_deserializer=LogReplayNode__pb2.decode_result.FromString,
                )
        self.OpenReplayFile = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/OpenReplayFile',
                request_serializer=Common__pb2.file_path.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.CloseReplayFile = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/CloseReplayFile',
                request_serializer=Common__pb2.file_path.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.GetReplayFileInfo = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/GetReplayFileInfo',
                request_serializer=Common__pb2.file_path.SerializeToString,
                response_deserializer=LogReplayNode__pb2.blf_file_info.FromString,
                )
        self.Reset = channel.unary_unary(
                '/LogReplayNode.LogReplayNode/Reset',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )


class LogReplayNodeServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartLog(self, request, context):
        """
        开始一个数据记录
        绝对路径
        log_request:
        - log_flag: 要启动记录的标志位
        - 0x01 << 0: CAN
        - 0x01 << 1: LIN
        result: 执行结果
        - 0: 启动记录成功
        - 1: 已经有一个记录任务在了
        - 2: 文件已经存在
        - 1000: raise
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetLogStatus(self, request, context):
        """
        获取当前的 Log 状态
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopLog(self, request, context):
        """
        停止一个数据记录任务
        result: 执行结果
        - 0: 结束成功
        - 1000: reise
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddLinDecodeRole(self, request, context):
        """
        添加 LIN 的解析规则
        generic_string: {
        'channels': list[int],    通道
        'ldfs': list[str],    通道对应的 ldf  文件
        }
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddCanDecodeRole(self, request, context):
        """
        添加 CAN 的解析规则
        generic_string: {
        'can_db_file_path': str,    CAN Arxml 文件
        'channels': list[int],    CAN 通道
        'cluster_names': list[str],    CAN 通道对应的 CAN_Cluster 名字
        'channel_id_filters': list[int],    要使能解包的 id
        }
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReplayNextN(self, request, context):
        """
        反序列化下面 n 个数据
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OpenReplayFile(self, request, context):
        """
        打开一个 blf 文件
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseReplayFile(self, request, context):
        """
        关闭当前的 blf 文件
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReplayFileInfo(self, request, context):
        """
        获取 blf 文件的描述信息
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Reset(self, request, context):
        """
        复位
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LogReplayNodeServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartLog': grpc.unary_unary_rpc_method_handler(
                    servicer.StartLog,
                    request_deserializer=LogReplayNode__pb2.log_request.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'GetLogStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetLogStatus,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.generic_string.SerializeToString,
            ),
            'StopLog': grpc.unary_unary_rpc_method_handler(
                    servicer.StopLog,
                    request_deserializer=LogReplayNode__pb2.log_request.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'AddLinDecodeRole': grpc.unary_unary_rpc_method_handler(
                    servicer.AddLinDecodeRole,
                    request_deserializer=Common__pb2.generic_string.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'AddCanDecodeRole': grpc.unary_unary_rpc_method_handler(
                    servicer.AddCanDecodeRole,
                    request_deserializer=Common__pb2.generic_string.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'ReplayNextN': grpc.unary_unary_rpc_method_handler(
                    servicer.ReplayNextN,
                    request_deserializer=LogReplayNode__pb2.decode_n.FromString,
                    response_serializer=LogReplayNode__pb2.decode_result.SerializeToString,
            ),
            'OpenReplayFile': grpc.unary_unary_rpc_method_handler(
                    servicer.OpenReplayFile,
                    request_deserializer=Common__pb2.file_path.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'CloseReplayFile': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseReplayFile,
                    request_deserializer=Common__pb2.file_path.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'GetReplayFileInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReplayFileInfo,
                    request_deserializer=Common__pb2.file_path.FromString,
                    response_serializer=LogReplayNode__pb2.blf_file_info.SerializeToString,
            ),
            'Reset': grpc.unary_unary_rpc_method_handler(
                    servicer.Reset,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'LogReplayNode.LogReplayNode', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class LogReplayNode(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartLog(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/StartLog',
            LogReplayNode__pb2.log_request.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetLogStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/GetLogStatus',
            Common__pb2.empty.SerializeToString,
            Common__pb2.generic_string.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopLog(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/StopLog',
            LogReplayNode__pb2.log_request.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddLinDecodeRole(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/AddLinDecodeRole',
            Common__pb2.generic_string.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddCanDecodeRole(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/AddCanDecodeRole',
            Common__pb2.generic_string.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReplayNextN(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/ReplayNextN',
            LogReplayNode__pb2.decode_n.SerializeToString,
            LogReplayNode__pb2.decode_result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OpenReplayFile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/OpenReplayFile',
            Common__pb2.file_path.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseReplayFile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/CloseReplayFile',
            Common__pb2.file_path.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReplayFileInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/GetReplayFileInfo',
            Common__pb2.file_path.SerializeToString,
            LogReplayNode__pb2.blf_file_info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Reset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/LogReplayNode.LogReplayNode/Reset',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from .ZoneSenderObject import ZoneSenderObject

class CanStackConfig(ZoneSenderObject):
    ''' CanStack 配置类
    '''
    def __init__(
        self, 
        channel: 'int',
        is_fd: 'bool',
        bitrate: 'int' = 500000,
        fd_bitrate: 'int' = 2000000,
        bus_type: 'str' = 'vector',
        app_name: 'str' = 'ZoneSender') -> None:
        super().__init__()
        self.channel = channel
        self.isFd = is_fd
        self.bitrate = bitrate
        self.fdBitrate = fd_bitrate
        self.busType = bus_type
        self.appName = app_name




# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import CanParserNode_pb2 as CanParserNode__pb2
from . import Common_pb2 as Common__pb2


class CanParserNodeStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetVersion = channel.unary_unary(
                '/CanParserNode.CanParserNode/GetVersion',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.version.FromString,
                )
        self.AddDbFile = channel.unary_unary(
                '/CanParserNode.CanParserNode/AddDbFile',
                request_serializer=CanParserNode__pb2.db_path.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.SetConfig = channel.unary_unary(
                '/CanParserNode.CanParserNode/SetConfig',
                request_serializer=CanParserNode__pb2.db_configs.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.Clear = channel.unary_unary(
                '/CanParserNode.CanParserNode/Clear',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.ClearSubscribe = channel.unary_unary(
                '/CanParserNode.CanParserNode/ClearSubscribe',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=Common__pb2.result.FromString,
                )
        self.GetMqttTopicTreeJson = channel.unary_unary(
                '/CanParserNode.CanParserNode/GetMqttTopicTreeJson',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=CanParserNode__pb2.mqtt_topic_tree.FromString,
                )
        self.EncodePdu = channel.unary_unary(
                '/CanParserNode.CanParserNode/EncodePdu',
                request_serializer=CanParserNode__pb2.i_signal_i_pdu_obj.SerializeToString,
                response_deserializer=CanParserNode__pb2.i_signal_i_pdu_encode.FromString,
                )
        self.GetCanDbInfo = channel.unary_unary(
                '/CanParserNode.CanParserNode/GetCanDbInfo',
                request_serializer=Common__pb2.empty.SerializeToString,
                response_deserializer=CanParserNode__pb2.can_db_info.FromString,
                )


class CanParserNodeServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetVersion(self, request, context):
        """获取版本
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddDbFile(self, request, context):
        """添加一个CANDB文件
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetConfig(self, request, context):
        """配置通道和CANDB的关联
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Clear(self, request, context):
        """清除所有CANDB文件
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearSubscribe(self, request, context):
        """清除所有订阅
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMqttTopicTreeJson(self, request, context):
        """获取所有的 Mqtt Tree json 字符串
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EncodePdu(self, request, context):
        """根据 PDU 的内容获取 PDU 的编码

        i_signal_i_pdu_encode.result 错误原因

        0         获取编码成功

        1000      raise Error

        1         找不到对应的通道

        2         找不到对应的 PDU 名字

        3         找不到 PDU context 中指定的信号名
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCanDbInfo(self, request, context):
        """
        获取当前 Can的信息
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CanParserNodeServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVersion,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.version.SerializeToString,
            ),
            'AddDbFile': grpc.unary_unary_rpc_method_handler(
                    servicer.AddDbFile,
                    request_deserializer=CanParserNode__pb2.db_path.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'SetConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.SetConfig,
                    request_deserializer=CanParserNode__pb2.db_configs.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'Clear': grpc.unary_unary_rpc_method_handler(
                    servicer.Clear,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'ClearSubscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearSubscribe,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=Common__pb2.result.SerializeToString,
            ),
            'GetMqttTopicTreeJson': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMqttTopicTreeJson,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=CanParserNode__pb2.mqtt_topic_tree.SerializeToString,
            ),
            'EncodePdu': grpc.unary_unary_rpc_method_handler(
                    servicer.EncodePdu,
                    request_deserializer=CanParserNode__pb2.i_signal_i_pdu_obj.FromString,
                    response_serializer=CanParserNode__pb2.i_signal_i_pdu_encode.SerializeToString,
            ),
            'GetCanDbInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCanDbInfo,
                    request_deserializer=Common__pb2.empty.FromString,
                    response_serializer=CanParserNode__pb2.can_db_info.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'CanParserNode.CanParserNode', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CanParserNode(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/GetVersion',
            Common__pb2.empty.SerializeToString,
            Common__pb2.version.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddDbFile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/AddDbFile',
            CanParserNode__pb2.db_path.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/SetConfig',
            CanParserNode__pb2.db_configs.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Clear(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/Clear',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearSubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/ClearSubscribe',
            Common__pb2.empty.SerializeToString,
            Common__pb2.result.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetMqttTopicTreeJson(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/GetMqttTopicTreeJson',
            Common__pb2.empty.SerializeToString,
            CanParserNode__pb2.mqtt_topic_tree.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EncodePdu(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/EncodePdu',
            CanParserNode__pb2.i_signal_i_pdu_obj.SerializeToString,
            CanParserNode__pb2.i_signal_i_pdu_encode.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCanDbInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/CanParserNode.CanParserNode/GetCanDbInfo',
            Common__pb2.empty.SerializeToString,
            CanParserNode__pb2.can_db_info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import typing
import copy
from .ZoneSenderObject import ZoneSenderObject

class LinMessage(ZoneSenderObject) :
    ''' LIN Message 对象，用来描述一个 CanFrame
    '''
    def __init__(self,
                    channel:int,
                    frameId:int,
                    dlc:int = 0,
                    data:typing.List[int] = [],
                    context:dict = dict(),
                    context_raw: dict = dict(),
                    ) -> None:
        super().__init__()
        self.frame_id = frameId
        self.frame_dlc = dlc
        self.frame_channel = channel
        self.frame_data = data
        self.context = copy.deepcopy(context)
        self.context_raw = copy.deepcopy(context_raw)
        self.name = str()

    def __str__(self) -> str:
        return 'frame_id: {0} data: {1}'.format(hex(self.frame_id).upper(), [hex(x_) for x_ in self.frame_data])


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProcessNode.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11ProcessNode.proto\x12\x0bProcessNode\x1a\x0c\x43ommon.proto\"+\n\x0breturn_info\x12\x0c\n\x04info\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\x05\x32\x87\x01\n\x0bProcessNode\x12;\n\x0eGetProcessInfo\x12\r.Common.empty\x1a\x18.ProcessNode.return_info\"\x00\x12;\n\x0eGetVersionInfo\x12\r.Common.empty\x1a\x18.ProcessNode.return_info\"\x00\x62\x06proto3')



_RETURN_INFO = DESCRIPTOR.message_types_by_name['return_info']
return_info = _reflection.GeneratedProtocolMessageType('return_info', (_message.Message,), {
  'DESCRIPTOR' : _RETURN_INFO,
  '__module__' : 'ProcessNode_pb2'
  # @@protoc_insertion_point(class_scope:ProcessNode.return_info)
  })
_sym_db.RegisterMessage(return_info)

_PROCESSNODE = DESCRIPTOR.services_by_name['ProcessNode']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RETURN_INFO._serialized_start=48
  _RETURN_INFO._serialized_end=91
  _PROCESSNODE._serialized_start=94
  _PROCESSNODE._serialized_end=229
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: SomeIpNode.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10SomeIpNode.proto\x12\nSomeIpNode\x1a\x0c\x43ommon.proto\"D\n\x0bsomeip_info\x12\x1e\n\x06result\x18\x01 \x01(\x0b\x32\x0e.Common.result\x12\x15\n\rjson_str_info\x18\x02 \x01(\t\"e\n\x0bservice_tag\x12\x14\n\x0cservice_name\x18\x01 \x01(\t\x12\x13\n\x0binstance_id\x18\x02 \x01(\x05\x12\x14\n\x0cservice_type\x18\x03 \x01(\t\x12\x15\n\rservice_state\x18\x04 \x01(\x08\";\n\x13someip_call_context\x12\x0f\n\x07timeout\x18\x02 \x01(\x05\x12\x13\n\x0bstr_context\x18\x01 \x01(\t\"N\n\x17someip_response_context\x12\x1e\n\x06result\x18\x01 \x01(\x0b\x32\x0e.Common.result\x12\x13\n\x0bstr_context\x18\x02 \x01(\t\"d\n\x13someip_service_info\x12\x14\n\x0cservice_name\x18\x01 \x01(\t\x12\x12\n\nservice_id\x18\x02 \x01(\x04\x12\x13\n\x0binstance_id\x18\x03 \x01(\r\x12\x0e\n\x06src_ip\x18\x04 \x01(\t\"f\n\x14someip_service_infos\x12\x1e\n\x06result\x18\x01 \x01(\x0b\x32\x0e.Common.result\x12.\n\x05infos\x18\x02 \x03(\x0b\x32\x1f.SomeIpNode.someip_service_info\"F\n\x11someip_arxml_json\x12\x0e\n\x06result\x18\x01 \x01(\x05\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\x11\n\tjson_data\x18\x03 \x01(\t2\x89\x07\n\nSomeIpNode\x12\x36\n\x10StartSomeIpStack\x12\x10.Common.net_info\x1a\x0e.Common.result\"\x00\x12\x32\n\x0fStopSomeIpStack\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12\x35\n\x0e\x41\x64\x64SomeIpArxml\x12\x11.Common.file_path\x1a\x0e.Common.result\"\x00\x12\x32\n\x0c\x41\x64\x64IfaceInfo\x12\x10.Common.net_info\x1a\x0e.Common.result\"\x00\x12\x31\n\x0cGetIfaceInfo\x12\r.Common.empty\x1a\x10.Common.net_info\"\x00\x12@\n\x0eGetArxmlToJson\x12\r.Common.empty\x1a\x1d.SomeIpNode.someip_arxml_json\"\x00\x12\x41\n\x15GetSomeIpServiceInfos\x12\r.Common.empty\x1a\x17.SomeIpNode.someip_info\"\x00\x12\x46\n\x19UpdateSomeipServiceConfig\x12\x17.SomeIpNode.service_tag\x1a\x0e.Common.result\"\x00\x12(\n\x05Reset\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12X\n\x0eSomeipCallSync\x12\x1f.SomeIpNode.someip_call_context\x1a#.SomeIpNode.someip_response_context\"\x00\x12\x37\n\x14GetSomeipStackStatus\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12G\n\x12GetAllOfferService\x12\r.Common.empty\x1a .SomeIpNode.someip_service_infos\"\x00\x12\x36\n\x08StartLog\x12\x18.Common.folder_file_path\x1a\x0e.Common.result\"\x00\x12\x35\n\x07StopLog\x12\x18.Common.folder_file_path\x1a\x0e.Common.result\"\x00\x12/\n\x0cGetLogStatus\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x62\x06proto3')



_SOMEIP_INFO = DESCRIPTOR.message_types_by_name['someip_info']
_SERVICE_TAG = DESCRIPTOR.message_types_by_name['service_tag']
_SOMEIP_CALL_CONTEXT = DESCRIPTOR.message_types_by_name['someip_call_context']
_SOMEIP_RESPONSE_CONTEXT = DESCRIPTOR.message_types_by_name['someip_response_context']
_SOMEIP_SERVICE_INFO = DESCRIPTOR.message_types_by_name['someip_service_info']
_SOMEIP_SERVICE_INFOS = DESCRIPTOR.message_types_by_name['someip_service_infos']
_SOMEIP_ARXML_JSON = DESCRIPTOR.message_types_by_name['someip_arxml_json']
someip_info = _reflection.GeneratedProtocolMessageType('someip_info', (_message.Message,), {
  'DESCRIPTOR' : _SOMEIP_INFO,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.someip_info)
  })
_sym_db.RegisterMessage(someip_info)

service_tag = _reflection.GeneratedProtocolMessageType('service_tag', (_message.Message,), {
  'DESCRIPTOR' : _SERVICE_TAG,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.service_tag)
  })
_sym_db.RegisterMessage(service_tag)

someip_call_context = _reflection.GeneratedProtocolMessageType('someip_call_context', (_message.Message,), {
  'DESCRIPTOR' : _SOMEIP_CALL_CONTEXT,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.someip_call_context)
  })
_sym_db.RegisterMessage(someip_call_context)

someip_response_context = _reflection.GeneratedProtocolMessageType('someip_response_context', (_message.Message,), {
  'DESCRIPTOR' : _SOMEIP_RESPONSE_CONTEXT,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.someip_response_context)
  })
_sym_db.RegisterMessage(someip_response_context)

someip_service_info = _reflection.GeneratedProtocolMessageType('someip_service_info', (_message.Message,), {
  'DESCRIPTOR' : _SOMEIP_SERVICE_INFO,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.someip_service_info)
  })
_sym_db.RegisterMessage(someip_service_info)

someip_service_infos = _reflection.GeneratedProtocolMessageType('someip_service_infos', (_message.Message,), {
  'DESCRIPTOR' : _SOMEIP_SERVICE_INFOS,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.someip_service_infos)
  })
_sym_db.RegisterMessage(someip_service_infos)

someip_arxml_json = _reflection.GeneratedProtocolMessageType('someip_arxml_json', (_message.Message,), {
  'DESCRIPTOR' : _SOMEIP_ARXML_JSON,
  '__module__' : 'SomeIpNode_pb2'
  # @@protoc_insertion_point(class_scope:SomeIpNode.someip_arxml_json)
  })
_sym_db.RegisterMessage(someip_arxml_json)

_SOMEIPNODE = DESCRIPTOR.services_by_name['SomeIpNode']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SOMEIP_INFO._serialized_start=46
  _SOMEIP_INFO._serialized_end=114
  _SERVICE_TAG._serialized_start=116
  _SERVICE_TAG._serialized_end=217
  _SOMEIP_CALL_CONTEXT._serialized_start=219
  _SOMEIP_CALL_CONTEXT._serialized_end=278
  _SOMEIP_RESPONSE_CONTEXT._serialized_start=280
  _SOMEIP_RESPONSE_CONTEXT._serialized_end=358
  _SOMEIP_SERVICE_INFO._serialized_start=360
  _SOMEIP_SERVICE_INFO._serialized_end=460
  _SOMEIP_SERVICE_INFOS._serialized_start=462
  _SOMEIP_SERVICE_INFOS._serialized_end=564
  _SOMEIP_ARXML_JSON._serialized_start=566
  _SOMEIP_ARXML_JSON._serialized_end=636
  _SOMEIPNODE._serialized_start=639
  _SOMEIPNODE._serialized_end=1544
# @@protoc_insertion_point(module_scope)



import typing
import copy
from .ZoneSenderObject import ZoneSenderObject

class SomeipPackage(ZoneSenderObject):
    ''' someip 发送或接收的数据

    :param service_name: str Someip服务名
    :param instance_id: int Someip Instance ID
    :param interface_name: str Someip 接口名
    :param interface_type: str 接口类型'method'|'get'|'set'|'notifer'|'event'
    :param context: dict 序列化后的数据 Dict
    :param payload: bytes 未序列化的原始 Bytes
    '''
    def __init__(
        self, 
        service_name: 'str' = '',
        instance_id: 'int' = -1,
        interface_name: 'str' = '',
        interface_type: 'str' = '',
        context: 'dict' = {},
        *args, **kwargs) -> None:
        #########################
        super().__init__()
        self.serviceName = service_name
        self.srcIp = kwargs.get('src_ip', '')
        self.srcPort = -1
        self.destIp = ''
        self.destPort = -1
        self.interfaceType = interface_type
        self.serviceId = kwargs.get('service_id', -1)
        self.instanceId = instance_id
        self.interfaceId = -1
        self.interfaceName = interface_name
        self.context = context
        self.payload = kwargs.get('payload', bytes())
        self.msgType = 0
        self.session_id = 0
    
    def __str__(self) -> str:
        # return super().__str__()
        # return '{0} {1} {2}'.format(self.serviceName, self.interfaceName, self.context)
        return \
        '''
srcIp: {0}
srcPort: {1}
destIp: {2}
destPort: {3}
type: {4}
serviceId: {5}
serviceName: {6}
instanceId: {7}
interfaceId: {8}
interfaceName: {9}
context: {10}
msgtype:{11}
session_id:{12}
        '''.format(
            self.srcIp,
            self.srcPort,
            self.destIp,
            self.destPort,
            self.interfaceType,
            self.serviceId,
            self.serviceName,
            self.instanceId,
            self.interfaceId,
            self.interfaceName,
            self.context,
            self.msgType,
            self.session_id,
        )

    @staticmethod
    def From(obj) -> 'SomeipPackage':
        ''' 从其他对象中构建 SomeipPackage
        :param obj: dict 表达 Someip 的 dict 对象
        '''
        if (isinstance(obj, dict)):
            someip_package_ = SomeipPackage(obj['sv_name'])
            someip_package_.srcIp = obj['src_ip']
            someip_package_.srcPort = obj['src_port']
            someip_package_.destIp = obj['dest_ip']
            someip_package_.destPort = obj['dest_port']
            someip_package_.interfaceType = obj['if_type']
            someip_package_.serviceId = obj['sv_id']
            someip_package_.instanceId = obj['ince_id']
            someip_package_.interfaceId = obj['if_id']
            someip_package_.interfaceName = obj['if_name']
            someip_package_.context = obj['context']
            someip_package_.payload = obj.get('payload', bytes())
            someip_package_.msgType = obj['msg_type']
            someip_package_.session_id = obj['session_id']
            return someip_package_
        raise TypeError

    def ToDict(self) -> 'dict':
        return {
            'sv_name': self.serviceName,
            'src_ip': self.srcIp,
            'src_port': self.srcPort,
            'dest_ip': self.destIp,
            'dest_port': self.destPort,
            'if_type': self.interfaceType,
            'sv_id': self.serviceId,
            'ince_id': self.instanceId,
            'if_id': self.interfaceId,
            'if_name': self.interfaceName,
            'context': self.context,
            'payload': self.payload,
            'msgType': self.msgType,
            'session_id':self.session_id,
        }

    @staticmethod
    def CopyData(someip_src: 'SomeipPackage', someip_dist: 'SomeipPackage') -> None:
        ''' 将 someip_src 的数据复制到 someip_dist 中
        '''
        someip_dist.serviceName = someip_src.serviceName
        someip_dist.srcIp = someip_src.srcIp
        someip_dist.srcPort = someip_src.srcPort
        someip_dist.destIp = someip_src.destIp
        someip_dist.destPort = someip_src.destPort
        someip_dist.interfaceType = someip_src.interfaceType
        someip_dist.serviceId = someip_src.serviceId
        someip_dist.instanceId = someip_src.instanceId
        someip_dist.interfaceId = someip_src.interfaceId
        someip_dist.interfaceName = someip_src.interfaceName
        someip_dist.context = someip_src.context
        someip_dist.payload = someip_src.payload


import copy
import ctypes
import random
import typing
import time,sys
from pathlib import Path
# FILE = Path(__file__).resolve()

# sys.path.append('.')
# sys.path.append(str(FILE.parent))
# sys.path.append(str(FILE.parent.parent))
from .BlfIoObjs import *




def BlfObjFromBytes(data:bytes) -> BlfIoStruct:
    # a_= data[0]
    obj_type_ = data[0]
    d_ = json.loads(data[1:].decode('utf-8'))
    try:
        del d_['_blfObjType']
    except:
        pass
    
    if (obj_type_ == BLF_ObjectType.CAN_FD_MESSAGE):
        return CanFdMessagePy(**d_)
    elif (obj_type_ == BLF_ObjectType.CAN_FD_MESSAGE_64):
        return CanFdMessage64Py(**d_)
    elif (obj_type_ == BLF_ObjectType.CAN_MESSAGE):
        return CanMessagePy(**d_)
    elif (obj_type_ == BLF_ObjectType.CAN_MESSAGE2):
        return CanMessage2Py(**d_)
    elif (obj_type_ == BLF_ObjectType.LIN_MESSAGE):
        return LinMessagePy(**d_)
    elif (obj_type_ == BLF_ObjectType.LIN_MESSAGE2):
        return LinMessage2Py(**d_)
    elif (obj_type_ == BLF_ObjectType.ETHERNET_FRAME):
        return EthernetFramePy(**d_)
    elif (obj_type_ == BLF_ObjectType.CAN_I_SIGNAL_I_PDU):
        return CanISignalIPduPy(**d_)
    elif (obj_type_ == BLF_ObjectType.ETH_SOMEIP_SD):
        return SomeIpSDFramePy(**d_)
    elif (obj_type_ == BLF_ObjectType.ETH_SOMEIP):
        return SomeIpFramePy(**d_)
    elif (obj_type_ == BLF_ObjectType.LIN_FRAME):
        return LinFramePy(**d_)
    else:
        return BlfIoStruct()


class BlfIo(object):
    def __init__(self, dll_path: str = '') -> None:
        '''
        blfIO 操作库
        :param dll_path:str dll 的路径，默认为本文件通路径下的 blf_io_export.dll
        '''
        if (dll_path == ''):
            # 如果是默认的 dll 路径
            dll_path = str(Path(__file__).absolute().parent / 'blf_io_export.dll')
        # print(dll_path)
        
        
        # 类内变量 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        self._log_start_ns = 0


        # Dll Obj >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        self._dll_blf_io_export = ctypes.cdll.LoadLibrary(dll_path)


        # Dll Function >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        self._dllFuncOpenReplayFile = self._dll_blf_io_export.OpenReplayFile
        self._dllFuncOpenReplayFile.argtypes = [ctypes.c_char_p]
        self._dllFuncOpenReplayFile.restype = ctypes.c_int

        self._dllFuncCloseReplayFile = self._dll_blf_io_export.CloseReplayFile
        self._dllFuncCloseReplayFile.restype = ctypes.c_int

        self._dllFuncReplayNext = self._dll_blf_io_export.ReplayNext
        # p_header_type, p_obj_type, p_time_stamp, p_time_stamp_type
        self._dllFuncReplayNext.argtypes = [ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32)]
        self._dllFuncReplayNext.restype = ctypes.c_int

        self._dllFuncOpenLogFile = self._dll_blf_io_export.OpenLogFile
        self._dllFuncOpenLogFile.argtypes = [ctypes.c_char_p]
        self._dllFuncOpenLogFile.restype = ctypes.c_int

        self._dllFuncCloseLogFile = self._dll_blf_io_export.CloseLogFile
        self._dllFuncCloseLogFile.restype = ctypes.c_int

        self._dllFuncLogNext = self._dll_blf_io_export.LogNext
        self._dllFuncLogNext.argtypes = [ctypes.c_uint32, ctypes.c_uint64]
        self._dllFuncLogNext.restype = ctypes.c_int
        

        # Dll Variable >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        # Header 结构体 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        self.OBJECT_HEADER_PY_REPLAY = ctypes.POINTER(ObjectHeaderPy)
        self.P_OBJECT_HEADER_PY_REPLAY = self.OBJECT_HEADER_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_OBJECT_HEADER_PY_REPLAY')
        self.OBJECT_HEADER_2_PY_REPLAY = ctypes.POINTER(ObjectHeader2Py)
        self.P_OBJECT_HEADER_2_PY_REPLAY = self.OBJECT_HEADER_2_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_OBJECT_HEADER_2_PY_REPLAY')

        # Replay 相关结构体 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        self.CAN_FD_MESSAGE_PY_REPLAY = ctypes.POINTER(CanFdMessagePy)
        self.P_CAN_FD_MESSAGE_PY_REPLAY = self.CAN_FD_MESSAGE_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_CAN_FD_MESSAGE_PY_REPLAY')
        self.CAN_FD_MESSAGE_64_PY_REPLAY = ctypes.POINTER(CanFdMessage64Py)
        self.P_CAN_FD_MESSAGE_64_PY_REPLAY = self.CAN_FD_MESSAGE_64_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_CAN_FD_MESSAGE_64_PY_REPLAY')
        self.CAN_MESSAGE_PY_REPLAY = ctypes.POINTER(CanMessagePy)
        self.P_CAN_MESSAGE_PY_REPLAY = self.CAN_MESSAGE_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_CAN_MESSAGE_PY_REPLAY')
        self.CAN_MESSAGE_2_PY_REPLAY = ctypes.POINTER(CanMessage2Py)
        self.P_CAN_MESSAGE_2_PY_REPLAY = self.CAN_MESSAGE_2_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_CAN_MESSAGE_2_PY_REPLAY')
        self.LIN_MESSAGE_PY_REPLAY = ctypes.POINTER(LinMessagePy)
        self.P_LIN_MESSAGE_PY_REPLAY = self.LIN_MESSAGE_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_LIN_MESSAGE_PY_REPLAY')
        self.LIN_MESSAGE_2_PY_REPLAY = ctypes.POINTER(LinMessage2Py)
        self.P_LIN_MESSAGE_2_PY_REPLAY = self.LIN_MESSAGE_2_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_LIN_MESSAGE_2_PY_REPLAY')
        self.ETHERNET_FRAME_PY_REPLAY = ctypes.POINTER(EthernetFramePy)
        self.P_ETHERNET_FRAME_PY_REPLAY = self.ETHERNET_FRAME_PY_REPLAY.in_dll(self._dll_blf_io_export, 'P_ETHERNET_FRAME_PY_REPLAY')

        # Log 相关结构体 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        self.CAN_FD_MESSAGE_PY_LOG = ctypes.POINTER(CanFdMessagePy)
        self.P_CAN_FD_MESSAGE_PY_LOG = self.CAN_FD_MESSAGE_PY_LOG.in_dll(self._dll_blf_io_export, 'P_CAN_FD_MESSAGE_PY_LOG')
        self.CAN_FD_MESSAGE_64_PY_LOG = ctypes.POINTER(CanFdMessage64Py)
        self.P_CAN_FD_MESSAGE_64_PY_LOG = self.CAN_FD_MESSAGE_64_PY_LOG.in_dll(self._dll_blf_io_export, 'P_CAN_FD_MESSAGE_64_PY_LOG')
        self.CAN_MESSAGE_PY_LOG = ctypes.POINTER(CanMessagePy)
        self.P_CAN_MESSAGE_PY_LOG = self.CAN_MESSAGE_PY_LOG.in_dll(self._dll_blf_io_export, 'P_CAN_MESSAGE_PY_LOG')
        self.CAN_MESSAGE_PY_2_LOG = ctypes.POINTER(CanMessage2Py)
        self.P_CAN_MESSAGE_2_PY_LOG = self.CAN_MESSAGE_PY_2_LOG.in_dll(self._dll_blf_io_export, 'P_CAN_MESSAGE_2_PY_LOG')
        self.LIN_MESSAGE_PY_LOG = ctypes.POINTER(LinMessagePy)
        self.P_LIN_MESSAGE_PY_LOG = self.LIN_MESSAGE_PY_LOG.in_dll(self._dll_blf_io_export, 'P_LIN_MESSAGE_PY_LOG')
        self.LIN_MESSAGE_PY_2_LOG = ctypes.POINTER(LinMessage2Py)
        self.P_LIN_MESSAGE_2_PY_LOG = self.LIN_MESSAGE_PY_2_LOG.in_dll(self._dll_blf_io_export, 'P_LIN_MESSAGE_2_PY_LOG')
        self.ETHERNET_FRAME_PY_LOG = ctypes.POINTER(EthernetFramePy)
        self.P_ETHERNET_FRAME_PY_LOG = self.ETHERNET_FRAME_PY_LOG.in_dll(self._dll_blf_io_export, 'P_ETHERNET_FRAME_PY_LOG')
    

    def OpenReplayFile(self, path: str) -> int:
        '''
        打开一个回放文件
        :param path: str 回放文件的路径
        :return: int
            - 0: 打开成功
            - 1: 打开失败 
            - 2: 文件不存在
        '''
        if (not Path(path).is_file()):
            return 1
        if (not Path(path).exists()):
            return 2
        res_ = self._dllFuncOpenReplayFile(path.encode('utf-8'))
        # print(type(res_))
        return res_

    def CloseReplyFile(self) -> int:
        '''
        关闭当前回放的文件
        :return: int\n
            - 0: 关闭成功\n
            - 1: 没有打开的回放文件
        '''
        return self._dllFuncCloseReplayFile()

    def ReplayNext(self) -> typing.Tuple[int, int, int, BlfIoStruct]:
        '''
        回放下一个报文
        :return:[0]int 是否读取成功
            - 0: 读取成功
            - 1: 文件损坏
            - 2: 下一条报文损坏
            - 3: 下一条报文是空指针
            - 4: 没有打开的文件
        :return:[1]int 时间戳
        :return:[2]int 时间戳类型
            - 0: 未知
            - 1: ms
            - 2: ns
        :return:[3]BlfIoStruct 下一个数据，可能为如下类型
            - CanFdMessagePy
            - CanFdMessage64Py
            - CanMessagePy
            - Canmessage2Py
            - LinMessagePy
            - LinMessage2Py
            - EthernetFramePy
        '''
        header_type_ = ctypes.c_uint32()    # 头类型
        obj_type_ = ctypes.c_uint32()    # 对象类型
        time_stamp_ = 0    # 时间戳数据
        time_stamp_type_ = 0    # 时间戳类型
        obj_ = BlfIoStruct()    # 返回的数据对象
        
        res_ = self._dllFuncReplayNext(ctypes.pointer(header_type_), ctypes.pointer(obj_type_))
        
        if (not (res_ == 0)):
            # 读取失败
            return res_, 0, 0, BlfIoStruct()

        # 检查 object_type
        # print('header_type = {0}'.format(header_type_.value))
        if (header_type_.value == 1):
            # ObjectHeader
            time_stamp_ = self.P_OBJECT_HEADER_PY_REPLAY.contents.objectTimeStamp
            time_stamp_type_ = self.P_OBJECT_HEADER_PY_REPLAY.contents.objectFlags
        elif (header_type_.value == 2):
            # ObjectHeader2
            time_stamp_ = self.P_OBJECT_HEADER_2_PY_REPLAY.contents.objectTimeStamp
            time_stamp_type_ = self.P_OBJECT_HEADER_2_PY_REPLAY.contents.objectFlags

        if (obj_type_.value == BLF_ObjectType.CAN_FD_MESSAGE):
            # obj_ = copy.deepcopy(self.P_CAN_FD_MESSAGE_PY_REPLAY.contents)
            obj_ = self.P_CAN_FD_MESSAGE_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.CAN_FD_MESSAGE

        if (obj_type_.value == BLF_ObjectType.CAN_FD_MESSAGE_64):
            # obj_ = copy.deepcopy(self.P_CAN_FD_MESSAGE_64_PY_REPLAY.contents)
            obj_ = self.P_CAN_FD_MESSAGE_64_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.CAN_FD_MESSAGE_64

        if (obj_type_.value == BLF_ObjectType.CAN_MESSAGE):
            # obj_ = copy.deepcopy(self.P_CAN_MESSAGE_PY_REPLAY.contents)
            obj_ = self.P_CAN_MESSAGE_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.CAN_MESSAGE

        if (obj_type_.value == BLF_ObjectType.CAN_MESSAGE2):
            # obj_ = copy.deepcopy(self.P_CAN_MESSAGE_2_PY_REPLAY.contents)
            obj_ = self.P_CAN_MESSAGE_2_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.CAN_MESSAGE2

        if (obj_type_.value == BLF_ObjectType.LIN_MESSAGE):
            # obj_ = copy.deepcopy(self.P_LIN_MESSAGE_PY_REPLAY.contents)
            obj_ = self.P_LIN_MESSAGE_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.LIN_MESSAGE

        if (obj_type_.value == BLF_ObjectType.LIN_MESSAGE2):
            # obj_ = copy.deepcopy(self.P_LIN_MESSAGE_2_PY_REPLAY.contents)
            obj_ = self.P_LIN_MESSAGE_2_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.LIN_MESSAGE2

        if (obj_type_.value == BLF_ObjectType.ETHERNET_FRAME):
            # obj_ = copy.deepcopy(self.P_ETHERNET_FRAME_PY_REPLAY.contents)
            obj_ = self.P_ETHERNET_FRAME_PY_REPLAY.contents
            obj_._blfObjType = BLF_ObjectType.ETHERNET_FRAME
            
        return res_, time_stamp_, time_stamp_type_, obj_

    def OpenLogFile(self, path: str) -> int:
        '''
        创建一个记录文件
        :param path: str 回放文件的路径
        :return: int\n
            - 0: 打开成功\n
            - 1: 已经有打开的文件了\n
            - 2: 打开文件失败 
        '''
        res_ = self._dllFuncOpenLogFile(path.encode('utf-8'))
        # print(type(res_))
        if (res_ == 0):
            self._log_start_ns = time.time_ns()
        return res_

    def CloseLogFile(self) -> int:
        '''
        关闭当前的记录文件
        :return: int\n
            - 0: 关闭成功\n
            - 1: 没有打开的回放文件
        '''
        return self._dllFuncCloseLogFile()

    def LogNext(self,obj: 'BlfIoStruct', time_stamp_ns: int = -1) -> int:
        '''
        记录一个 Frame
        :return: int\n
            - 0: 记录成功\n
            - 1: 文件未打开\n
            - 10: 未知的数据结构
        '''
        time_stamp_ns_ = time_stamp_ns
        if (time_stamp_ns_ < 0):
            time_stamp_ns_ = time.time_ns() - self._log_start_ns
        else:
            time_stamp_ns_ = time_stamp_ns_ >> 16
        if (isinstance(obj, CanFdMessagePy)):
            self.P_CAN_FD_MESSAGE_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.CAN_FD_MESSAGE), ctypes.c_uint64(time_stamp_ns_))
        if (isinstance(obj, CanFdMessage64Py)):
            self.P_CAN_FD_MESSAGE_64_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.CAN_FD_MESSAGE_64), ctypes.c_uint64(time_stamp_ns_))
        if (isinstance(obj, CanMessagePy)):
            self.P_CAN_MESSAGE_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.CAN_MESSAGE), ctypes.c_uint64(time_stamp_ns_))
        if (isinstance(obj, CanMessage2Py)):
            self.P_CAN_MESSAGE_2_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.CAN_MESSAGE2), ctypes.c_uint64(time_stamp_ns_))
        if (isinstance(obj, LinMessagePy)):
            self.P_LIN_MESSAGE_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.LIN_MESSAGE), ctypes.c_uint64(time_stamp_ns_))
        if (isinstance(obj, LinMessage2Py)):
            self.P_LIN_MESSAGE_2_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.LIN_MESSAGE2), ctypes.c_uint64(time_stamp_ns_))
        if (isinstance(obj, EthernetFramePy)):
            self.P_ETHERNET_FRAME_PY_LOG.contents = obj
            return self._dllFuncLogNext(ctypes.c_uint32(BLF_ObjectType.ETHERNET_FRAME), ctypes.c_uint64(time_stamp_ns_))
        return 10



from typing import Union
import json
import typing

import grpc

from . import ZoneSenderData
from .ObjIo import *

from .Protos import SomeIpNode_pb2, SomeIpNode_pb2_grpc


class SomeipNodeClient(object):
    def __init__(self) -> None:
        """ SomeipNode 的客户端
        """
        self._someipNodeStub = SomeIpNode_pb2_grpc.SomeIpNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'
                    .format(
                        ZoneSenderData.SOMEIP_STACK_NODE_IP, 
                        ZoneSenderData.SOMEIP_STACK_NODE_PORT),
                options = ZoneSenderData.GRPC_OPTIONS
            )
        )

    def StartLogging(self,folder_path:str,file_name:str) -> 'int':
        ''' 启动 记录以太网数据
        :param folder_path: str 文件夹
        :param file_name: str 文件名字
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.StartLog(
                SomeIpNode_pb2.Common__pb2.folder_file_path(
                    folder = folder_path,
                    file = file_name,
                )
            )
            if res_.result == 0 :
                return 0
            else :
                raise Exception(res_.reason)
        except Exception as e_ :
            print(e_)
            return 1000
    
    def StopLogging(self,folder_path:str,file_name:str) -> 'int':
        ''' 停止 记录以太网数据
        :param folder_path: str 文件夹
        :param file_name: str 文件名字
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.StopLog(
                SomeIpNode_pb2.Common__pb2.folder_file_path(
                    folder = folder_path,
                    file = file_name,
                )
            )
            if res_.result == 0 :
                return 0
            else :
                raise Exception(res_.reason)
        except Exception as e_ :
            print(e_)
            return 1000
    
    def GetLoggingStatus(self) -> Union[tuple,int]:
        ''' 获取记录数据状态
        :return: tuple\n
            - tuple: 成功 tuple[0] 1为有任务，0是没有任务；tuple[1] 为当前记录文件名\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.GetLogStatus(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            return (res_.result,res_.reason)
        except Exception as e_ :
            print(e_)
            return 1000

    def StartSomeIpStack(self, ip_addr: 'str', iface: 'str') -> 'int':
        ''' 启动 SomeIp 协议栈

        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.StartSomeIpStack(
                SomeIpNode_pb2.Common__pb2.net_info(
                    ip_addr = ip_addr,
                    iface = iface
                )
            )
            print('启动Someip协议栈, result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def StopSomeIpStack(self) -> 'int':
        '''关闭 SomeIp 协议栈

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.StopSomeIpStack(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            print('关闭SomeIp协议栈, result: {0}, reason:{1}'.format(res_.result, res_.reason))
            return res_.reason
        except Exception as e_:
            print(e_)
            return 1000

    def AddSomeIpArxml(self, arxml_path: 'str') -> 'int':
        ''' 添加一个 SomeIp Arxml 文件, 可以重复调用添加多个

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.AddSomeIpArxml(
                SomeIpNode_pb2.Common__pb2.file_path(
                    path = arxml_path
                )
            )
            print('加载 SomeIp Arxml 文件 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.reason
        except Exception as e_:
            print(e_)
            return 1000
    
    def AddIfaceInfo(self,ip:str,ifacename:str) -> 'int':
        ''' 添加一个网卡信息, 可以重复添加，但是只会记录最后一个，当前只支持单网卡模式
        :param ip: str 网卡的IPv4地址
        :param ip: str 网卡的名字
        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.AddIfaceInfo(
                SomeIpNode_pb2.Common__pb2.net_info(
                    ip_addr = ip,
                    iface = ifacename
                )
            )
            if res_.result == 0 :
                return 0
            else :
                raise Exception(res_.reason)
        except Exception as e_ :
            print(e_)
            return 1000
    
    def GetIfaceInfo(self) -> Union['tuple','int'] :
        ''' 获取网卡的信息

        :return: int | tuple

            - tuple: 成功,元组为（ip,name）

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.GetIfaceInfo(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            if res_.ip_addr and res_.iface :
                return (res_.ip_addr,res_.iface)
            else :
                raise Exception('获取网卡信息失败')
        except Exception as e_ :
            print(e_)
            return 1000
 
    def GetArxmlJsonData(self) -> Union[dict,int] :
        ''' 获取所有 SomeIp Arxml 文件中的信息

        :return: int | dict

            - dict: 成功,字典里为arxml信息

            - 1000: error
        '''
        try:
            res_ = self._someipNodeStub.GetArxmlToJson(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            if res_.result == 0 :
                return json.loads(res_.json_data)
            else :
                raise Exception(f'{res_.reason}')
        except Exception as e_ :
            print(e_)
            return 1000

    def GetSomeIpServiceInfos(self, return_d: 'dict') -> 'int':
        ''' 获取当前已经加载的 SomeIp Arxml Info

        :param return_d: 如果获取成功，将会把获取的信息填充到该字典中
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.GetSomeIpServiceInfos(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            if (not res_.result.result == 0):
                return res_.result.result
            return_d.clear()
            return_d.update(json.loads(res_.json_str_info))
            return res_.result.result
        except Exception as e_:
            print(e_)
            return 1000

    def UpdateSomeIpServiceConfig(
        self, 
        service_name: 'str', 
        instance_id: 'int', 
        service_type: 'str',
        service_state : 'bool' = True) -> 'int':
        ''' 更新SomeIp中服务的信息

        :param service_id: str SomeIp Service ID
        :param instance_id: int SomeIp Service Instance ID
        :param service_type: str 可以是 'consumer'|'provider' 表示该服务设置为什么类型
        :param state : int 默认值是True，用户可以不用填，如果需要停止服务，该参数赋值False

        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.UpdateSomeipServiceConfig(
                SomeIpNode_pb2.service_tag(
                    service_name = service_name,
                    instance_id = instance_id,
                    service_type = service_type,
                    service_state = service_state,
                )
            )
            print('更新 SomeIp 服务设定 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def Reset(self) -> 'int':
        ''' 复位 SomeIp 协议栈，并清空 SomeIp 服务配置

        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._someipNodeStub.Reset(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            print('复位 SomeIp 协议栈 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def GetSomeIpStackStatus(self) -> 'int':
        '''获取当前 Someip Stack 的状态

        :return: int\n
            - 0 正在运行\n
            - 1 协议栈未启动\n
            - 2 协议栈未初始化\n
            - 1000 error
        '''
        try:
            res_ = self._someipNodeStub.GetSomeipStackStatus(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            print('SomeipStack 的状态为 {0}'.format(res_.result))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def SomeipCallSync(
        self, 
        someip_package_in: 'SomeipPackage', 
        someip_package_out: 'SomeipPackage', 
        timeout: 'int' = 1000,
        *args, **kwargs) -> 'int':
        ''' 同步调用 SomeipCall
        :param someip_package_in: 要调用的 SomeipPackage
        :param someip_package_out: 如果成功调用，则将返回的 SomeipPackage 填充到该对象中
        :param timeout: 超时参数，单位 ms
        :param by: str 设置使用什么来发送 Someip 'context'|'payload'
        :return: int\n
            - 0 正在运行\n
            - 1 超时\n
            - 1000 error
        '''
        try:
            d_ = someip_package_in.ToDict()
            d_.update({'by': kwargs.get('by', 'context')})
            d_.update({'payload': d_['payload'].hex()})
            res_ = self._someipNodeStub.SomeipCallSync(
                SomeIpNode_pb2.someip_call_context(
                    timeout = timeout,
                    str_context = json.dumps(d_)
                )
            )
            result_ = res_.result.result
            if (result_ == 0):
                # 成功回复
                recv_d_ = json.loads(res_.str_context)
                recv_d_.update({'payload': bytes.fromhex(recv_d_['payload'])})
                # print(recv_d_)
                # someip_package_out = SomeipPackage.From(recv_d_)
                SomeipPackage.CopyData(SomeipPackage.From(recv_d_), someip_package_out)
                return 0
            else:
                print(res_.result.reason)
                return result_
        except Exception as e_:
            print(e_)
            return 1000

    def GetAllOfferService(self, offer_services_out:list) -> int:
        '''
        获取当前 SomeipStack 所有的 Offer 的服务\n
        注意，如果获取成功，offer_service 会清空\n
        :param offer_service:list 获取到的服务信息将传到该 list 中\n
        :return:int
            - 0:获取成功
            - 1:协议栈未启动
            - 1000:error
        '''
        try:
            res_ = self._someipNodeStub.GetAllOfferService(
                SomeIpNode_pb2.Common__pb2.empty()
            )
            if (res_.result.result != 0):
                print(res_.result.reason)
                return res_.result.result
            offer_services_out.clear()
            for info_ in res_.infos:
                offer_services_out.append(SomeipPackage(
                    service_name=info_.service_name,
                    instance_id=info_.instance_id,
                    service_id=info_.service_id,
                    src_ip=info_.src_ip,
                ))
            print('GetAllOfferService result {0}'.format(res_.result.result))
            return res_.result.result
        except Exception as e_:
            print('GetAllOfferService Except {0}'.format(e_))
            return 1000


    


import threading

class TestAssist(object):
    def __init__(self) -> None:
        self.TA_TEXT_EVENT_DICT = dict()

    def TA_WaitForTextEvent(self, text: 'str', time_out_ms: 'int') -> 'int':
        ''' 等待字符串, 如果在规定时间等不到就 pass

        :param text: str 要等待的字符串
        :param time_out_ms: int 等待的时间
        :return: int\n
            - 0 在规定时间等到了期望的字符串\n
            - 1 超时, 在规定的时间没有等到期望的字符串\n
            - 2 线程拿不到 Event 对象\n
        '''
        if (text not in self.TA_TEXT_EVENT_DICT.keys()):
            self.TA_TEXT_EVENT_DICT[text] = threading.Event()
        event_ = self.TA_TEXT_EVENT_DICT[text]
        if (not isinstance(event_, threading.Event)):
            return 2
        event_.clear()
        if (not event_.wait(time_out_ms/1000)):
            return 1
        return 0

    def TA_TrigerTextEvent(self, text: 'str') -> None:
        ''' 触发字符串事件

        :param text: str 要触发的字符串，触发后会让所有的等待字符串事件通过
        '''
        if (text not in self.TA_TEXT_EVENT_DICT.keys()):
            return
        event_ = self.TA_TEXT_EVENT_DICT[text]
        if (not isinstance(event_, threading.Event)):
            return
        event_.set()


import typing
import copy
from .ZoneSenderObject import ZoneSenderObject


class CanMessage(ZoneSenderObject):
    ''' CanMessage 对象，用来描述一个 CanMessage
    '''
    def __init__(
        self,
        arbitration_id: 'int',
        channel: 'int',
        dlc: 'int' = 0,
        data: typing.List['int'] = [],
        is_fd: 'bool' = False,
        is_extended_id: 'bool' = False,
        is_remote_frame: 'bool' = False,
        context: 'dict' = dict(),
        ) -> None:
        ...
        # #####################################
        super().__init__()
        self.arbitration_id = arbitration_id
        self.is_extended_id = is_extended_id
        self.is_remote_frame = is_remote_frame
        self.channel = channel
        self.dlc = dlc
        self.data = data
        self.is_fd = is_fd
        self.context = copy.deepcopy(context)

    def __str__(self) -> str:
        # return super().__str__()
        return 'arbitration_id: {0} data: {1}'.format(hex(self.arbitration_id).upper(), [hex(x_) for x_ in self.data])



# array_ = bytearray()
# data_ = 0xFFFFFFFFFFFFFFF0
# bytes_ = data_.to_bytes(8, 'big', signed=False)
# array_.extend(bytes_)
# print(bytes_)

import os
import typing
import random
import time
import json
import csv

from multiprocessing import RLock, Process, Lock

from SharedMemZoneSender import SharedMemZoneSender

def BuildRandomData():
    d_ = {}
    l_ = []
    for i_ in range(10000):
        time_ = i_
        data_ = [random.randint(0, 0xFF)] * 1
        l_.append({
            'time': time_,
            'data': data_,
        })
    d_['data'] = l_

    file_name_ = 'random_data_' + time.strftime('%H%M%S') + '.json'
    with open(file_name_, mode='w+') as f_:
        f_.write(json.dumps(d_, indent=2))

def GetData() -> typing.List[dict]:
    # file_ = open('random_data_104015.json', mode='r')    # 10000
    file_ = open('random_data_103822.json', mode='r')    # 100
    d_ = json.loads(file_.read())
    return d_['data']

def ManagerFunction(rlock):
    shared_ = SharedMemZoneSender(rlock, create=True)
    while (True):
        time.sleep(5)

def ProviderFunction(rlock):
    shared_ = SharedMemZoneSender(rlock)
    data_ = GetData()
    print('Process:{0} 开始写入时间 {1}'.format(os.getpid(), time.time()))
    for i_ in data_:
        time_stamp_ns_ = i_['time']
        # print(time_stamp_ns_)
        data_ = i_['data']
        shared_.WriteNext(time_stamp_ns_, bytes(data_))
        time.sleep(0.001)
    print('Process:{0} 结束写入时间 {1}'.format(os.getpid(), time.time()))

def ProviderFunction2(rlock):
    shared_ = SharedMemZoneSender(rlock)
    # data_ = GetData()
    print('Process:{0} 开始写入时间 {1}'.format(os.getpid(), time.time()))
    # for i_ in range(10000):
    while (True):
        time_stamp_ns_ = time.time_ns()
        # print(time_stamp_ns_)
        # data_ = bytes([random.randint(0, 0xFF)] * 512)
        # data_ = bytes([1, 3, 0, 0, 0, 0])
        data_ = bytes([0, 0, 0])
        shared_.WriteNext(time_stamp_ns_, bytes(data_))
        time.sleep(0.001)
    print('Process:{0} 结束写入时间 {1}'.format(os.getpid(), time.time()))


def ConsumerFunction(rlock, name:int):
    shared_ = SharedMemZoneSender(rlock)
    # time.sleep(10)
    print('开始读数据 ConsumerFunction >>>>>')
    with open('Consumer_{0}.csv'.format(name), 'w', newline='') as f:
        writer = csv.writer(f)
        total_counter_ = 0
        while(True):
            time_stamp_ns_, data_ = shared_.ReadNext()
            if (len(data_) != 0):
                # total_counter_ += 1
                # print('写入, {0}, {1}'.format(time_stamp_ns_, list(data_)))
                writer.writerow((time_stamp_ns_, list(data_)))
                # print('已写入, {0}'.format(total_counter_))

def ConsumerFunction2(rlock, name:int):
    shared_ = SharedMemZoneSender(rlock)
    # time.sleep(10)
    print('开始读数据 ConsumerFunction2 >>>>>')
    with open('Consumer_{0}.csv'.format(name), 'w', newline='') as f:
        writer = csv.writer(f)
        total_counter_ = 0
        while(True):
            time.sleep(0.02)    # 100ms 读一次
            bytes_list_ = shared_.ReadToLatest()
            if (len(bytes_list_) != 0):
                # print(len(bytes_list_))
                total_counter_ += len(bytes_list_)
                writer.writerows(bytes_list_)
                print('Process, {0}, 已写入, {1}'.format(name, total_counter_))
        
def TestCsv():
    with open('some.csv', 'w', newline='') as f:
        writer = csv.writer(f)
        while(True):
            # time_stamp_ns_, data_ = shared_.ReadNext()
            writer.writerow([time.time_ns(), bytes([0, 1, 2,6, 4])])
            time.sleep(0.1)

# def 

def TestMutilProcessIO():
    rlock_ = Lock()

    manager_process_ = Process(target=ManagerFunction, args=[rlock_], daemon=True)

    provider_process_1_ = Process(target=ProviderFunction2, args=[rlock_], daemon=True)
    provider_process_2_ = Process(target=ProviderFunction2, args=[rlock_], daemon=True)
    provider_process_3_ = Process(target=ProviderFunction2, args=[rlock_], daemon=True)
    provider_process_4_ = Process(target=ProviderFunction2, args=[rlock_], daemon=True)
    provider_process_5_ = Process(target=ProviderFunction2, args=[rlock_], daemon=True)
    provider_process_6_ = Process(target=ProviderFunction2, args=[rlock_], daemon=True)

    consumer_process_1_ = Process(target=ConsumerFunction2, args=[rlock_, 1], daemon=True)
    consumer_process_2_ = Process(target=ConsumerFunction2, args=[rlock_, 2], daemon=True)
    consumer_process_3_ = Process(target=ConsumerFunction2, args=[rlock_, 3], daemon=True)
    consumer_process_4_ = Process(target=ConsumerFunction2, args=[rlock_, 4], daemon=True)
    consumer_process_5_ = Process(target=ConsumerFunction2, args=[rlock_, 5], daemon=True)
    consumer_process_6_ = Process(target=ConsumerFunction2, args=[rlock_, 6], daemon=True)

    manager_process_.start()

    time.sleep(1)
    provider_process_1_.start()
    # provider_process_2_.start()
    # provider_process_3_.start()
    # provider_process_4_.start()
    # provider_process_5_.start()
    # provider_process_6_.start()

    time.sleep(1)
    consumer_process_1_.start()
    # consumer_process_2_.start()
    # consumer_process_3_.start()
    # consumer_process_4_.start()
    # consumer_process_5_.start()
    # consumer_process_6_.start()

    while(True):
        time.sleep(5)

def LockAcq(lock, name:int):
    # while(True)
    mem_ = SharedMemZoneSender(lock)
    print('进程 {0} 准备拿锁'.format(name))
    mem_.rlock.acquire(True)
    print('进程 {0} 拿到锁'.format(name))
    time.sleep(10)
    mem_.rlock.release()
    print('进程 {0} 释放锁'.format(name))

# def LockAcq2(lock):
#     mem_ = SharedListZoneSender(lock)
#     print('进程 2 准备拿锁')
#     mem_.rlock.acquire(True)
#     print('进程 2 拿到锁')
#     time.sleep(10)
#     mem_.rlock.release()
#     print('进程 2 释放锁')

def TestLock():
    rlock_ = Lock()

    manager_process_ = Process(target=ManagerFunction, args=[rlock_], daemon=True)

    provider_process_1_ = Process(target=LockAcq, args=[rlock_, 1], daemon=True)
    provider_process_2_ = Process(target=LockAcq, args=[rlock_, 2], daemon=True)
    provider_process_3_ = Process(target=LockAcq, args=[rlock_, 3], daemon=True)

    manager_process_.start()
    time.sleep(1)

    provider_process_1_.start()
    time.sleep(1)
    provider_process_2_.start()
    time.sleep(1)
    provider_process_3_.start()

    while(True):
        time.sleep(5)

if __name__ == '__main__':
    # BuildRandomData()

    TestMutilProcessIO()

    # TestLock()


'''
测试BlfOb 读数据
'''

from BlfIoObjs import *
from BlfIo import BlfObjFromBytes, BlfIo
import random
import time
from multiprocessing import Process

import threading

def TestReadFile():
    blfio_ = BlfIo()
    res_ = blfio_.OpenReplayFile(r"Z:\log_1659518748.8437243.blf")
    counter_ = 0
    counter_tag_ = 10000
    counter_bash_ = 0
    if (res_ == 0):
        while(True):
            is_succ, time_, time_type, obj_ = blfio_.ReplayNext()
            if (is_succ != 0):
                break
            print('Time, {0}, obj, {1}'.format(time_, obj_.ToDict()))
            counter_ += 1
            if (counter_ % counter_tag_ == 0):
                counter_bash_ += 1
                print('读了 {0} 个 10000 条'.format(counter_bash_)) 
        blfio_.CloseReplyFile()

if __name__ == '__main__':
    task1_ = Process(target=TestReadFile, daemon=True)
    task2_ = Process(target=TestReadFile, daemon=True)
    task1_.start()
    # task2_.start()

    while (True):
        time.sleep(5)


from typing import Union
from typing import List
import grpc
from . import ZoneSenderData
from .ObjIo import *

from .Protos import LinStackNode_pb2,LinStackNode_pb2_grpc


class LinStackNodeClient(object) :
    """
    LinStackNode 的客户端
    """
    def __init__(self) -> None:
        self._linStackStub = LinStackNode_pb2_grpc.LinStackNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'
                    .format(
                        ZoneSenderData.LIN_STACK_NODE_IP, 
                        ZoneSenderData.LIN_STACK_NODE_PORT),
                options = ZoneSenderData.GRPC_OPTIONS
            )
        )

    def setConfig(self,configs:List[dict]) -> int:

        ''' LIN Stack设置硬件类型,目前只支持vector
        
        :param configs: List[dict] 
            configs example:\n
            [{'hardwareType':'vector','appName':'zoneSender'},{'hardwareType':'pcan','appName':'zoneSender'}]\n

        :return: int\n
            - 0: 成功\n
            - 1: 配置失败\n
            - 1000: error\n
        '''

        try:
            configs_ = list()
            for config_ in configs :
                configs_.append(
                    LinStackNode_pb2.lin_stack_config(
                        hardwareType = config_['hardwareType'],
                        appName = config_['appName'],
                    )
                )
            res_ = self._linStackStub.SetConfig(
                LinStackNode_pb2.lin_stack_configs(
                    config = configs_,
                )
            )
            print('setConfig result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def startLinStack(self) -> int:
        ''' 启动LIN Stack
        :return: int\n
            - 0: 成功\n
            - 1: 未设置LIN Stack,需要先setConfig\n
            - 2: LIN Stack已经在运行\n
            - 3: LIN Stack启动失败
            - 1000: error\n
        '''
        try:
            res_ = self._linStackStub.StartLinStack(
                LinStackNode_pb2.Common__pb2.empty()
            )
            print('StartLinStack result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def stopLinStack(self) -> int:
        '''停止LIN Stack
        :return: int\n
            - 0: 成功\n
            - 1: 未设置LIN Stack,需要先setConfig\n
            - 2: LIN Stack不在运行\n
            - 3: LIN Stack停止失败
            - 1000: error\n
        '''
        try:
            res_ = self._linStackStub.StopLinStack(
                LinStackNode_pb2.Common__pb2.empty()
            )
            print('StopLinStack result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000

    def reset(self) -> int:
        '''重置LIN Stack，会清除LIN Stack所有信息，需要重新设置然后启动才能正常使用
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._linStackStub.Reset(
                LinStackNode_pb2.Common__pb2.empty()
            )
            print('ResetLinStack result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000

    def setMessageSimulation(self,channel:int,id:int,simu:bool) -> int:
        '''设置LIN报文响应仿真
        :param channel: int 设置的LIN通道
        :param id: int 指定报文的ID
        :param simu: bool True为使能仿真，False为关闭仿真
        :return: int\n
            - 0: 成功\n
            - 1: 未设置LIN Stack,需要先setConfig\n
            - 2: LIN Stack未运行\n
            - 3: LIN 驱动层错误
            - 1000: error\n
        '''
        try:
            res_ = self._linStackStub.SetMessageSimulation(
                LinStackNode_pb2.lin_message_config(
                    id = id,
                    simu = simu,
                    channel = channel,
                )
            )
            print('setMessageSimulation result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def setHeaderSimulation(self,channel:int,simu:bool) -> int:
        '''设置主节点报文头仿真
        :param channel: int 设置的LIN通道
        :param simu: bool True为使能仿真，False为关闭仿真
        :return: int\n
            - 0: 成功\n
            - 1: 未设置LIN Stack,需要先setConfig\n
            - 2: LIN Stack未运行\n
            - 3: 当前通道不是主节点模式，不支持主节点报文头仿真
            - 4: LIN 驱动层错误
            - 1000: error\n
        '''
        try:
            res_ = self._linStackStub.SetHeaderSimulation(
                LinStackNode_pb2.lin_header_config(
                    simu = simu,
                    channel = channel,
                )
            )
            print('setHeaderSimulation result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def clearSubscribe(self) -> int:
        '''清除LIN Stack中所有的订阅
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._linStackStub.ClearSubscribe(
                LinStackNode_pb2.Common__pb2.empty()
            )
            print('clearSubscribe result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def isRunning(self) -> Union[int,bool]:
        '''获取LIN Stack当前的运行状态
        :return: int or bool\n
            - True: 运行\n
            - False: 不在运行\n
            - 1000 : error
        '''
        try:
            res_ = self._linStackStub.GetStatus(
                LinStackNode_pb2.Common__pb2.empty()
            )
            if res_.status == 0:
                return True
            else:
                return False
        except Exception as e_ :
            print(e_)
            return 1000


import os
import typing

import grpc

from . import ZoneSenderData
from .ObjIo import *

from .Protos import CanStackNode_pb2, CanStackNode_pb2_grpc


class CanStackNodeClient(object):
    def __init__(self) -> None:
        """ CanStackNode的客户端
        """
        self._canStub = CanStackNode_pb2_grpc.CanStackNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'
                    .format(
                        ZoneSenderData.CAN_STACK_NODE_IP, 
                        ZoneSenderData.CAN_STACK_NODE_PORT),
                options = ZoneSenderData.GRPC_OPTIONS
            )
        )

    def SetConfigs(
        self,
        configs
        ) -> 'int':
        """ 设置 CAN 协议线的参数

        :param configs: typing.List[ZoneSender.ObjIo.CanStackConfig]
        :return: int

            - 0: 成功

            - 1000: error
        """
        try:
            configs_ = list()
            for config_ in configs:
                configs_.append(
                    CanStackNode_pb2.can_channel_config(
                        channel = config_.channel,
                        bitrate = config_.bitrate,
                        is_fd = config_.isFd,
                        fd_bitrate = config_.fdBitrate,
                        bus_type = config_.busType,
                        app_name = config_.appName,
                    )
                )
            res_ = self._canStub.SetConfigs(
                CanStackNode_pb2.can_channel_configs(
                    configs = configs_
                )
            )
            print('SetCanStackConfig result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def StartCanStack(self) -> 'int':
        '''启动 CAN 协议栈

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._canStub.StartCanStack(CanStackNode_pb2.Common__pb2.empty())
            print('StartCanStack result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def StopCanStack(self) -> 'int':
        ''' 关闭Can协议栈

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._canStub.StopCanStack(CanStackNode_pb2.Common__pb2.empty())
            print('StopCanStack result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def ClearSend(self) -> 'int':
        '''关闭所有CAN循环任务

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._canStub.ClearSend(CanStackNode_pb2.Common__pb2.empty())
            print('ClearCanCycTask result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def ClearSubscribe(self) -> 'int':
        ''' 清空订阅的客户端列表

        :return: int

            - 0: 成功

            - 1000: error
        '''
        try:
            res_ = self._canStub.ClearSubscribe(
                CanStackNode_pb2.Common__pb2.empty()
            )
            print('清除所有 CanStack 的订阅 result: {0}, reason: {0}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def StartLog(
        self, 
        name: 'str', 
        file_path: 'str', 
        channels: typing.List['int'],
        max_log_time_minute: 'int' = 60) -> 'int':
        ''' 启动一个记录任务

        :param name: str 该记录任务的名称，用于唯一标识一个记录任务
        :param file_path: str 要记录的文件存放的位置，必须注明记录的文件类型，如 xxx.blf
        :param channels: List['int'] 要记录的通道列表
        :max_log_time_minute: int 最大记录的分钟数 单位 min
        :return: int

            - 0: 成功

            - 1: 失败，未知的文件类型

            - 1000: error
        '''
        try:
            abs_file_path_ = os.path.abspath(file_path)
            res_ = self._canStub.StartLog(
                CanStackNode_pb2.log_start_request(
                    name = name,
                    file_path = abs_file_path_,
                    max_log_time_minute = max_log_time_minute,
                    channels = channels,
                )
            )
            print('StartLog reasut: {0} reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def StopLog(
        self, 
        name: 'str') -> 'int':
        '''停止一个记录任务

        :param name: str 该记录任务的名称，传入之前启动记录任务使用的名称
        :return: int

            - 0: 成功

            - 1: 失败，找不到指定的记录任务

            - 1000: error
        '''
        try:
            res_ = self._canStub.StopLog(
                CanStackNode_pb2.log_stop_request(
                    name = name,
                )
            )
            print('StopLog result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000

    def ClearLogger(self) -> 'int':
        ''' 取消所有的记录任务

        :return: int
            - 0: 成功
            - 1: 失败，找不到指定的记录任务
            - 1000: error
        '''
        try:
            res_ = self._canStub.ClearLogger(CanStackNode_pb2.Common__pb2.empty())
            print('清除所有数据记录任务 result: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print(e_)
            return 1000


    def SetCrcRcConfig(self, channel: int, arbitration_id: int, crc_bit_starts: list, rc_bit_starts: list, crc_table: list) -> 'int':
        '''
        设置 CAN CRC 和 RC 的配置
        设置以后，指定的 CAN 报文将会在指定的 bit 位置设置 CRC 和 RC
        :param channel: int 要配置的通道
        :param arbitration_id: int 要配置的 CAN ID
        :param crc_bit_starts: list[int]  CRC 数据场的起始 bit 位
        :param rc_bit_starts: list[int] RC 数据场的起始 bit 位
        :crc_table: list[int] 要计算的 CRC TABLE
        '''
        try:
            res_ = self._canStub.SetCrcRcConfig(CanStackNode_pb2.crc_rc_config(
                channel = channel,
                arbitration_id = arbitration_id,
                crc_bit_starts = crc_bit_starts,
                rc_bit_starts = rc_bit_starts,
                crc_table = crc_table,
            ))
            print('CanStackNode 配置 CRC RC Config reason: {0}, reason: {1}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_:
            print('CanStackNode 配置 CRC RC Config result: {0}, reason: {1}'.format(1000, str(e_)))
            return 1000

    def CreatTimeEvent(self, timer):
        res_ = self._canStub.CreatTimerEvent(CanStackNode_pb2.timer(timer_cycle_time=timer))
        print(f'设置回调定时器周期{timer},设置结果{res_.result}')
        return res_.result

    def GetStackStatus(self):
        res_ = self._canStub.GetStackStatus(CanStackNode_pb2.Common__pb2.empty())
        print(f'当前canstack状态为{res_.result}')
        return res_.result





import json
import os
import typing

from .CanParserNodeClient import CanParserNodeClient
from .CanStackNodeClient import CanStackNodeClient
from .SomeipNodeClient import SomeipNodeClient
from .LinParseNodeClient import LinParseNodeClient
from .LinStackNodeClient import LinStackNodeClient
from .LogReplayNodeClient import LogReplayNodeClient


class ZoneSenderFramework(object):
    '''
    ZoneSender 的基础类，可以用来实现对硬件的简单控制\n
    该对象有几个成员变量，可以查看相应的对象介绍页面来了解函数列表:\n
        - CanStack: ZoneSender.CanStackNodeClient\n
        - CanParser: ZoneSender.CanParserNodeClient\n
        - SomeipStack: ZoneSender.SomeipNodeClient\n
    通过调用几个成员变量的函数实现各种控制功能\n
    '''
    def __init__(self) -> None:
        super().__init__()
        self.CanStack = CanStackNodeClient()
        self.CanParser = CanParserNodeClient()
        self.SomeipStack = SomeipNodeClient()
        self.LinStack = LinStackNodeClient()
        self.LinParser = LinParseNodeClient()
        self.LogReplay = LogReplayNodeClient()


    def Reset(self) -> None:
        ''' 复位所有功能，包括:\n
            - 复位 CAN 协议栈\n
            - 清空 CAN DB 文件\n
            - 清除所有数据记录任务\n
            - 清空 SomeipStack \n
            - 清空 SomeipStack Arxml 文件\n
            - 复位 LIN 协议栈\n
            - 清空 LIN ldf文件\n

        '''
        self.CanStack.StopCanStack()
        self.CanStack.ClearSubscribe()
        self.CanStack.ClearSend()
        self.CanStack.ClearLogger()
        self.CanParser.ClearSubscribe()
        self.CanParser.ClearCanDb()
        self.SomeipStack.Reset()
        self.LinStack.reset()
        self.LinParser.clearChannelConfig()
        self.LinParser.clearSubscribe()

    def Reset_LIN(self) ->None :
        ''' 复位所有功能，包括:\n
            - 复位 LIN 协议栈\n
            - 清空 LIN ldf文件\n
        '''
        self.LinStack.reset()
        self.LinParser.clearChannelConfig()
        self.LinParser.clearSubscribe()

    def Reset_CAN(self) ->None :
        ''' 复位所有功能，包括:\n
            - 复位 CAN 协议栈\n
            - 清空 CAN DB 文件\n
            - 清除所有数据记录任务\n
        '''
        self.CanStack.StopCanStack()
        self.CanStack.ClearSubscribe()
        self.CanStack.ClearSend()
        self.CanStack.ClearLogger()
        self.CanParser.ClearSubscribe()
        self.CanParser.ClearCanDb()
    
    def Reset_SomeIp(self) -> None :
        ''' 复位所有功能，包括:\n
            - 清空 SomeipStack \n
            - 清空 SomeipStack Arxml 文件\n
        '''
        self.SomeipStack.Reset()
        





from .CanFrame import CanFrame
from .CanMessage import CanMessage
from .CanSignal import CanSignal
from .CanISignalPdu import CanISignalIPdu
from .CanStackConfig import CanStackConfig
from .SomeipPackage import SomeipPackage
from .LinFrame import LinFrame
from .LinMessage import LinMessage
from .LinSignal import LinSignal


import json
import typing
from pathlib import Path

import grpc

from . import ZoneSenderData
from .BlfIo import BlfIoObjs, BlfObjFromBytes
from .ObjIo import *
from .Protos import LogReplayNode_pb2, LogReplayNode_pb2_grpc
from .SharedMemory import IndexNode, SharedMemZoneSender


class LogReplayNodeClient(object):
    def __init__(self) -> None:
        '''
        ZoneSender 用于记录和回放的客户端
        '''
        self._logReplayStub = LogReplayNode_pb2_grpc.LogReplayNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'.format(ZoneSenderData.LOG_REPLAY_NODE_IP, ZoneSenderData.LOG_REPLAY_NODE_PORT),
                options=ZoneSenderData.GRPC_OPTIONS
            )
        )
        self._shareMemReplayData = SharedMemZoneSender(None, name='ZoneSenderReplay')

    def AddLinDecodeRole(
        self, 
        channels: typing.List[int],
        ldfs: typing.List[str],
        **kwargs) -> int:
        '''
        添加一个 LIN 数据的解析规则
        可以重复添加
        :param channels:list[int] LIN 通道列表
        :param ldfs:list[int] ldf 文件列表
        :return:int
            - 0: 添加成功
            - 1000: raise
        '''
        try:
            ldfs = [str(Path(x_).absolute()) for x_ in ldfs]
            res_ = self._logReplayStub.AddLinDecodeRole(LogReplayNode_pb2.Common__pb2.generic_string(
                text = json.dumps({
                    'channels': channels,
                    'ldfs': ldfs,
                })
            ))
            # res_ = self._logReplayStub.AddLinDecodeRole(LogReplayNode_pb2.lin_decode_role(
            #     channels = channels,
            #     ldfs = ldfs,
            # ))
            print(res_.reason)
            return res_.result
        except Exception as e_:
            print('LogReplayNodeClient.AddLinDecodeRole Exception reason {0}'.format(e_))
            return 1000

    def AddCanDecodeRole(
        self,
        can_db_file_path:str,
        channels:typing.List[int],
        cluster_names:typing.List[str],
        channel_id_filters:typing.List[typing.List[int]],
        **kwargs
        ) -> int:
        '''
        添加一个 CAN 数据的解析规则
        可以重复添加
        :param can_db_file_path:str CAN DB 文件的路径
        :param channels:list[int] CAN 通道列表
        :param cluster_names:list[str] CAN 通道对应的 CAN_Cluster 名字
        :param channel_id_filters:list[int] CAN 通道对应的 要使能解包的 ID
        :return:int
            - 0: 添加成功
            - 1000: raise
        '''
        try:
            # res_ = self._logReplayStub.AddCanDecodeRole(LogReplayNode_pb2.can_decode_role(
            #     can_db_file_path = str(Path(can_db_file_path).absolute()),
            #     channels = channels,
            #     cluster_names = cluster_names,
            # ))
            res_ = self._logReplayStub.AddLinDecodeRole(LogReplayNode_pb2.Common__pb2.generic_string(
                text = json.dumps({
                    'can_db_file_path': str(Path(can_db_file_path).absolute()),
                    'channels': channels,
                    'cluster_names': cluster_names,
                    'channel_id_filters': channel_id_filters,
                })
            ))
            print(res_.reason)
            return res_.result
        except Exception as e_:
            print('LogReplayNodeClient.AddCanDecodeRole Exception reason {0}'.format(e_))
            return 1000

    def Reset(self) -> int:
        '''
        复位
            - 清空解析配置文件
            - 关闭所有正在解析的文件
        :return:int
            - 0: 复位成功
            - 1000: raise
        '''
        try:
            res_ = self._logReplayStub.Reset(LogReplayNode_pb2.Common__pb2.empty())
            print(res_.reason)
            return res_.result
        except Exception as e_:
            print('LogReplayNodeClient.Reset Exception reason {0}'.format(e_))
            return 1000

    def ReplayNextN(
        self, 
        blf_file_path: str, 
        count: int, 
        result_l_obj: typing.List[BlfIoObjs.BlfIoStruct], 
        reslut_l_time_stamp: typing.List[int]) -> int:
        '''
        解包 N 个数据
        :param blf_file_path:str 要读的 Blf 文件名
        :param count:int 要解析下面的多少个数据 不能超过 131000
        :param result_l:list[BlfIoObjs.BlfIoStruct] 解析后的数据 里面会放 BlfIoObjs.CanISignalIPduPy | BlfIoObjs.LinFramePy
        :return:int
            - 0:成功
            - 1:读结束
        '''
        try:
            res_ = self._logReplayStub.ReplayNextN(LogReplayNode_pb2.decode_n(
                blf_file_path = blf_file_path,
                count = count,
                )
            )
            result_ = int(res_.result.result)    # 结果
            reason_ = str(res_.result.reason)    # 原因
            start_index_ = int(res_.start_index)    # 起始 index
            stop_index_ = int(res_.stop_index)    # 结束 index
            is_stop_ = bool(res_.is_stop)    # 是否读完了
            
            # 写数据 >>>>>>>>
            result_l_obj.clear()
            reslut_l_time_stamp.clear()
            for i_ in range(start_index_, stop_index_):
                node_ = self._shareMemReplayData.GetIndexNode(i_)
                blf_obj_ = BlfObjFromBytes(self._shareMemReplayData.ReadFrame(i_))
                reslut_l_time_stamp.append(node_.time_stamp_ns)
                result_l_obj.append(blf_obj_)
                # pass
            print(res_.result.reason)
            return res_.result.result
            # return 1
        except Exception as e_:
            print('LogReplayNodeClient.DecodeNextN Exception reason {0}'.format(e_))
            return 1000

    def OpenReplayFile(self, blf_file_path: str) -> int:
        '''
        打开回放的文件
        :param blf_file_path:str 要打开的文件路径
        :return:int
            - 0:成功
            - other:失败
        '''
        try:
            res_ = self._logReplayStub.OpenReplayFile(
                LogReplayNode_pb2.Common__pb2.file_path(
                    path=blf_file_path,
                )
            )
            print(res_.reason)
            return res_.result
        except Exception as e_:
            print('LogReplayNodeClient.OpenReplayFile Exception reason {0}'.format(e_))
            return 1000

    def CloseReplayFile(self, blf_file_path: str) -> int:
        '''
        关闭回放的文件
        :param blf_file_path:str 要关闭的文件路径
        :return:int
            - 0:成功
            - other:失败
        '''
        try:
            res_ = self._logReplayStub.CloseReplayFile(
                LogReplayNode_pb2.Common__pb2.file_path(
                    path=blf_file_path,
                )
            )
            print(res_.reason)
            return res_.result
        except Exception as e_:
            print('LogReplayNodeClient.CloseReplayFile Exception reason {0}'.format(e_))
            return 1000

    def GetReplayFileInfo(self, blf_file_path: str, file_info:dict) -> int:
        '''
        获取文件信息
        :param blf_file_path:str 要关闭的文件路径
        :param file_info:dict 文件信息的描述
        :return:int
            - 0:成功
            - other:失败
        '''
        try:
            return 0
        except Exception as e_:
            print('LogReplayNodeClient.GetReplayFileInfo Exception reason {0}'.format(e_))
            return 1000

    def StartLog(self,filepath:str,proto:str) -> tuple:
        '''
        开始录制数据
        :param filepath:str 要录制的文件路径
        :param proto:str 录制的协议，比如CAN/LIN
        :return:tuple (结果,原因)
        '''
        try:
            res_ = self._logReplayStub.StartLog(LogReplayNode_pb2.log_request(
                file_path = LogReplayNode_pb2.Common__pb2.file_path(
                    path = filepath
                ),
                log_flag = 0x1 if proto == 'can' else 0x2 ,
            ))
            if res_.result == 0 :
                return (0,'开始记录成功')
            else :
                raise Exception(f'{res_.reason}')
        except Exception as e_ :
            return (1000,e_)

    def StopLog(self,filepath:str,proto:str) -> int:
        '''
        结束录制数据
        :param filepath:str 要关闭的文件路径
        :param proto:str 录制的协议，比如CAN/LIN
        :return:int
            - 0:成功
            - 1000:失败
        '''
        try:
            res_ = self._logReplayStub.StopLog(LogReplayNode_pb2.log_request(
                file_path = LogReplayNode_pb2.Common__pb2.file_path(
                    path = filepath
                ),
                log_flag = 0x1 if proto == 'can' else 0x2 ,
            ))
            if res_.result == 0 :
                return (0,'停止记录成功')
            else :
                print(res_.reason)
                raise Exception(f'{res_.reason}')
        except Exception as e_ :
            print(e_)
            return (1000,e_)

    def GetLogStatus(self) -> int:
        return 0


from typing import Union
import grpc
import json

from . import ZoneSenderData
from .ObjIo import *

from .Protos import LinParserNode_pb2,LinParserNode_pb2_grpc

class LinParseNodeClient(object) :
    """
    LinParserNode 的客户端
    """
    def __init__(self) -> None:
        self._linDbParserStub = LinParserNode_pb2_grpc.LinParserNodeStub(
            channel=grpc.insecure_channel(
                target='{0}:{1}'
                    .format(
                        ZoneSenderData.LIN_PARSERNODE_NODE_IP, 
                        ZoneSenderData.LIN_PARSERNODE_NODE_PORT),
                options = ZoneSenderData.GRPC_OPTIONS
            )
        )
    
    def setChannelConfig(self,appChannel:int,ldfPath:str,linMode:str,
                            txrecv:int = 0,baudrate:int = 0) -> int:
        ''' 添加一个ldf文件到特定的通道上
        :param appChannel: int vector硬件配置上对应的LIN通道 
        :param ldfPath: str ldf文件的路径
        :param linMode: str 该通道的模式，支持Master,Slave 
        :param txrecv: int 默认全接收，0为全接收，1为接收收到的报文，2为接收发送的报文
        :param baudrate: int 默认19200,波特率
        :return: int\n
            - 0: 成功\n
            - 1: 配置失败\n
            - 1000: error\n
        '''
        try:
            res_ = self._linDbParserStub.SetChannelConfig(
                LinParserNode_pb2.lin_channel_config(
                    ldf_path = ldfPath,
                    lin_mode = linMode,
                    txrecv = txrecv,
                    baudrate = baudrate,
                    lin_channel = appChannel,
                )
            )
            print('setChannelConfig result: {0}, reason: {0}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def getLdfJson(self) -> Union[dict,int]:
        ''' 获取配置的ldf，并返回解析后的ldf
        :return: dict,int\n
            - dict: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._linDbParserStub.GetLdfJsonTree(
                LinParserNode_pb2.Common__pb2.empty()
            )
            print('getLdfJson result: {0}, reason: {0}'.format(res_.result, res_.reason))
            if res_.result == 0 :
                return json.loads(res_.json_data)
            else:
                raise Exception(f'{res_.reason}')
        except Exception as e_ :
            print(e_)
            return 1000

    def clearChannelConfig(self) ->int:
        ''' 清空所有已配置的ldf文件
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._linDbParserStub.ClearDbfile(
                LinParserNode_pb2.Common__pb2.empty()
            )
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000
    
    def clearSubscribe(self) :
        ''' 清空所有的订阅
        :return: int\n
            - 0: 成功\n
            - 1000: error\n
        '''
        try:
            res_ = self._linDbParserStub.ClearSubscribe(
                LinParserNode_pb2.Common__pb2.empty()
            )
            print('清除所有 LinParser 的订阅 result: {0}, reason: {0}'.format(res_.result, res_.reason))
            return res_.result
        except Exception as e_ :
            print(e_)
            return 1000

    def setFrameSimulation(self,channel:int,frame:Union[int,str],simu:bool) -> int:
        ''' 设置LIN报文仿真
        :param channel: int vector硬件配置上对应的LIN通道
        :param frame: int or str LIN报文的名字或者ID
        :param simu: bool True为仿真，False为关闭仿真
        :return: int\n
            - 0: 成功\n
            - 1: 未找到指定报文
            - 1000: error\n
        '''
        try:
            if isinstance(frame,str) :
                linframe = LinParserNode_pb2.lin_frame_config(
                    frame_name = frame,
                    channel = channel,
                    simu = simu,
                )
            elif isinstance(frame,int) :
                linframe = LinParserNode_pb2.lin_frame_config(
                    frame_id = frame,
                    channel = channel,
                    simu = simu,
                )
            else :
                print(f'frame type unsupport,input type is {type(frame)}')
                return 1000
            res_ = self._linDbParserStub.SetFrameSimulation(linframe)
            print('setFrameSimulation result: {0}, reason: {0}'.format(res_.result, res_.reason))
            return res_.result
        except Exception  as e_ :
            print(e_)
            return 1000

    def setNodeSimulation(self,channel:int,Node:str,simu:bool) -> int:
        ''' 设置LIN节点仿真
        :param channel: int vector硬件配置上对应的LIN通道
        :param Node: str LIN节点的名字
        :param simu: bool True为仿真，False为关闭仿真
        :return: int\n
            - 0: 成功\n
            - 1: 未找到指定的节点
            - 1000: error\n
        '''
        try:
            if isinstance(Node,str) :
                res_ = self._linDbParserStub.SetNodeSimulation(
                    LinParserNode_pb2.lin_node_config(
                        node_name = Node,
                        channel = channel,
                        simu = simu,
                    )
                )
            else :
                print(f'Node type unsupport,input type is {type(Node)}')
                return 1000
            print('setNodeSimulation result: {0}, reason: {0}'.format(res_.result, res_.reason))
            return res_.result
        except Exception  as e_ :
            print(e_)
            return 1000

        




import ctypes
import logging
import typing
import os
import time
from multiprocessing import Lock, shared_memory

DELAY_TIME_INSERT = 100000000    # 插入数据会过滤 100 ms 以前的数据
DELAY_TIME_VISABLE = 300000000    # 显示会滞后 300 ms


class IndexNode(ctypes.Structure):
    _fields_ = [
        ('prev', ctypes.c_uint64),
        ('next', ctypes.c_uint64),
        ('current', ctypes.c_uint64),
        ('time_stamp_ns', ctypes.c_uint64),
        ('valid', ctypes.c_uint8),
        ]

    def __init__(self, prev:int, next:int, current:int, time_stamp_ns:int, valid:int) -> None:
        '''
        双向链表节点数据结构
        :param prev:int 前一个 Node 的索引
        :param next:int 后一个 Node 的索引
        :param current:int 当前 Node 的索引
        :param time_stamp:int 当前节点的时间戳
        :param valid:int 当前节点是否是有效的
            - 0: 无效
            - 1: 有效
        '''
        self.prev = prev
        self.next = next
        self.current = current
        self.time_stamp_ns = time_stamp_ns
        self.valid = valid

    def ToBytes(self) -> 'bytes':
        '''
        转换为 bytes
        :return:bytes 该Node的Bytes表达
        '''
        prev_ = self.prev.to_bytes(length=8, byteorder='big', signed=False)
        next_ = self.next.to_bytes(length=8, byteorder='big', signed=False)
        current_ = self.current.to_bytes(length=8, byteorder='big', signed=False)
        time_stamp_ = self.time_stamp_ns.to_bytes(length=8, byteorder='big', signed=False)
        valid_ = self.valid.to_bytes(length=1, byteorder='big', signed=False)
        return prev_ + next_ + current_ + time_stamp_ + valid_

class SharedMemZoneSender(object):
    DATA_SIZE = 16384
    INDEX_SIZE = 32
    def __init__(self, lock=None, name: str='ZoneSender', create: bool=False, size:int=1024*128) -> None:
        '''
        ZoneSender 的共享内存类, 可以实现高性能的数据跨进程 IO
        :param lock:multiprocessing.Lock 写进程的互斥锁
        :param name:str 共享内存块的名字
        :param create:bool 是否要创建新的内存块，默认行情况下只能由管理进程创建
        :param size:int 要创建的内存块长度 默认是 128 MB
        '''
        self._logger = logging.getLogger('ShareMemNode')
        self._logger.debug('test1')
        if (create):
            self.sharedDataListMem = shared_memory.ShareableList(sequence=[bytes([0xFF]* SharedMemZoneSender.DATA_SIZE)]*size, name=name+'Data')
            self.sharedIndexListMem = shared_memory.ShareableList(sequence=[bytes([0xFF]* SharedMemZoneSender.INDEX_SIZE)]*size, name=name+'Index')
            self.latestIndexMem = shared_memory.SharedMemory(name=name+'LatestIndex', create=True, size=8)
            self.firstIndexMem = shared_memory.SharedMemory(name=name+'FirstIndex', create=True, size=8)
            self.readyWriteIndexMem = shared_memory.SharedMemory(name=name+'ReadyWriteIndex', create=True, size=8)
            self.isNewMem = shared_memory.SharedMemory(name=name+'IsNew', create=True, size=1)
            self._SetFirstIndex(0)
            self._SetLatsetIndex(0)
            self._SetReadyWriteIndex(0)
            self._SetIsNew(1)
        else:
            self.sharedDataListMem = shared_memory.ShareableList(name=name+'Data')
            self.sharedIndexListMem = shared_memory.ShareableList(name=name+'Index')
            self.latestIndexMem = shared_memory.SharedMemory(name=name+'LatestIndex', create=False)
            self.firstIndexMem = shared_memory.SharedMemory(name=name+'FirstIndex', create=False)
            self.readyWriteIndexMem = shared_memory.SharedMemory(name=name+'ReadyWriteIndex', create=False)
            self.isNewMem = shared_memory.SharedMemory(name=name+'IsNew', create=False)
        self.lock = lock
        self.size = size
        self.readTime = time.time_ns()
        self.Sync()

    def Sync(self) -> None:
        '''
        同步到最新的数据
        '''
        self.nextReadIndex = self._GetLatestIndex()
        self.isReadToLatest = True
    
    def WriteNext(self, time_stamp_ns:int, data:'bytes') -> None:
        '''
        写下一帧数据
        :param time_stamp:int 下一帧的时间戳
        :param data:bytes 下一帧的数据 不能超过 1024 个 bytes
        '''
        # lock_ = RLock().acquire()
        self.lock.acquire(True)
        # try:

        try:
            ready_write_index_ = self._GetReadyWriteIndex()
            first_index_ = self._GetFirstIndex()
            latest_index_ = self._GetLatestIndex()

            # 如果是新的，直接写一个，然后返回
            if (self._GetIsNew() == 1):
                index_node_ = IndexNode(0,0,0,time_stamp_ns, 1)
                self.WriteFrame(0, data)
                # self.WriteIndex(index_node_)
                self._SetIsNew(0)
                self.WriteIndex(index_node_)
                self._SetReadyWriteIndex(1)
                return

            # 时间戳对比
            index_node_ = self.GetIndexNode(latest_index_)
            index_node_next_ = index_node_
            # 如果要插入的数据时间早于最新时间 DELAY_TIME_INSERT 就返回
            if ((index_node_.time_stamp_ns - DELAY_TIME_INSERT) > time_stamp_ns):
                return
            
            # find_counter_ = 0
            # while(index_node_.time_stamp_ns > time_stamp_ns):
            #     find_counter_ += 1
            #     index_node_ = self.GetIndexNode(index_node_.prev)
            #     index_node_next_ = self.GetIndexNode(index_node_.next)

            # 写数据
            # 如果是最新的数据
        # try:
            # if (find_counter_ == 0):
                # 直接在最后面追加
                # 将当前最新的下一个改为即将写的
            prev_node_ = IndexNode(index_node_.prev, ready_write_index_, index_node_.current, index_node_.time_stamp_ns, 1)    # 前一帧的指针
            self.WriteIndex(prev_node_)
            current_node_ = IndexNode(index_node_.current, ready_write_index_, ready_write_index_, time_stamp_ns, 1)    # 当前这一帧的指针
            self.WriteIndex(current_node_)
            latest_index_ = ready_write_index_
            # else:
            #     # 如果要写的时间戳向前 pop 了
            #     prev_node_ = IndexNode(index_node_.prev, ready_write_index_, index_node_.current, index_node_.time_stamp_ns, 1)    # 要插入的前一个指针
            #     self.WriteIndex(prev_node_)
            #     next_node_ = IndexNode(ready_write_index_, index_node_next_.next, index_node_next_.current, index_node_next_.time_stamp_ns, 1)    # 要插入的后一个指针
            #     self.WriteIndex(next_node_)
            #     current_node_ = IndexNode(index_node_.current, index_node_next_.current, ready_write_index_, time_stamp_ns, 1)    # 当前这一帧的指针
            #     self.WriteIndex(current_node_)

            # 写节点
            self.WriteFrame(ready_write_index_, data)

            # 修改下一个要写的 index
            ready_write_index_ += 1
            if (ready_write_index_ >= self.size):
                self._SetReadyWriteIndex(0)
            else:
                self._SetReadyWriteIndex(ready_write_index_)

            # if (find_counter_ == 0):
            self._SetLatsetIndex(latest_index_)
        except Exception as e_:
            # print('WriteNext Exception {0} {1}'.format(type(e_), e_))
            self._logger.error('WriteNext Exception {0} {1}'.format(type(e_), e_))
        finally:
            self.lock.release()

            # return
        # except Exception as e_:
        #     print('WriteNext {0}'.format(e_))
        # finally:
        # self.lock.release()

    def ReadNext(self) -> typing.Tuple[int, bytes]:
        '''
        读下一帧数据
        :return:tuple 返回的数据
            - [0]:int 下一帧的时间戳
            - [1]:bytes 下一帧的数据
        '''
        time_now_ns_ = time.time_ns()
        index_node_ = self.GetIndexNode(self.nextReadIndex)
        
        if (index_node_.valid != 1):
            return (0, bytes())

        if ((index_node_.time_stamp_ns + DELAY_TIME_VISABLE) > time_now_ns_):
        # if ((index_node_.time_stamp_ns) > time_now_ns_):
            # print('========')
            return (0, bytes())

        if (index_node_.current == index_node_.next):
            # 如果本次要读的数据是最新的数据
            if (self.isReadToLatest):
                # 如果已经读到最新了
                return (0, bytes())
            else:
                self.isReadToLatest = True
                return (index_node_.time_stamp_ns, self.ReadFrame(index_node_.current))
        else:
            # 如果本次要读的数据不是最新的数据
            self.isReadToLatest = False
            self.nextReadIndex = index_node_.next
            return (index_node_.time_stamp_ns, self.ReadFrame(index_node_.current))

    def ReadToLatest(self) -> typing.List[typing.Tuple[int, bytes]]:
        '''
        读到最新时刻
        :return:List[bytes] 到最新时刻的数据
        '''
        l_ = []
        while (True):
            time_, data_ = self.ReadNext()
            if (len(data_) > 0):
                l_.append((time_, data_))
            else:
                break
        return l_

    def ReadFrame(self, index:int) -> 'bytes':
        '''
        读指定位置的数据
        :param index:int 指定的位置索引
        :return:bytes 指定位置的数据
        '''
        length_ = int.from_bytes(self.sharedDataListMem[index][0:8], byteorder='big', signed=False)
        # node_ = self.GetIndexNode(index)
        data_ = self.sharedDataListMem[index][8:8+length_]
        act_len_ = len(data_)
        if ((act_len_ == 0) and (length_ != 0)):
            # 如果实际数据是空，但是长度不为空，就返回指定长度的 0 bytes
            a_= bytes([0]*length_)
            # print(a_)
            return a_
        if (act_len_ != length_):
            # 如果得到的 bytes 数据长度和期望长度不同，就在尾部补充 /x00
            l_ = list(data_)
            l_.extend([0]*(length_ - act_len_))
            data_ = bytes(l_)
        return data_

    def WriteFrame(self, index:int, data:'bytes') -> None:
        '''
        写指定位置的数据
        :param index:int 要写入的位置
        :param data:bytes 要写入的数据
        '''
        self.sharedDataListMem[index] = len(data).to_bytes(8, byteorder='big', signed=False) + data

    def _SetLatsetIndex(self, index:int) -> None:
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        self.latestIndexMem.buf[0:8] = index.to_bytes(length=8, byteorder='big', signed=False)
    
    def _GetLatestIndex(self) -> int:
        '''
        获取最新位置的索引
        :return:int 最新位置的索引
        '''
        return int.from_bytes(self.latestIndexMem.buf[0:8], byteorder='big', signed=False)

    def _SetFirstIndex(self, index:int) -> None:
        '''
        设置第一个位置的索引
        :param index:int 要设置的第一个位置的索引
        '''
        self.firstIndexMem.buf[0:8] = index.to_bytes(length=8, byteorder='big', signed=False)
    
    def _GetFirstIndex(self) -> int:
        '''
        获取第一个位置的索引
        :return:int 第一个位置的索引
        '''
        return int.from_bytes(self.firstIndexMem.buf[0:8], byteorder='big', signed=False)

    def _SetReadyWriteIndex(self, index:int) -> None:
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        self.readyWriteIndexMem.buf[0:8] = index.to_bytes(length=8, byteorder='big', signed=False)
    
    def _GetReadyWriteIndex(self) -> int: 
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        return int.from_bytes(self.readyWriteIndexMem.buf[0:8], byteorder='big', signed=False)

    def _SetIsNew(self, index:int) -> None:
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        self.isNewMem.buf[0:1] = index.to_bytes(length=1, byteorder='big', signed=False)
    
    def _GetIsNew(self) -> int:
        '''
        获取是否最新内存
        :return: int
            - 0:不是新内存
            - 1:是新内存
        '''
        return int.from_bytes(self.isNewMem.buf[0:1], byteorder='big', signed=False)

    def WriteIndex(self, index_node: IndexNode) -> None:
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        self.sharedIndexListMem[index_node.current] = index_node.ToBytes()

    def GetIndexNode(self, index:int) -> IndexNode:
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        if (index > self.size):
            print('GetIndexNode(self, index:int) {0}'.format(index))
        data_ = self.sharedIndexListMem[index]
        len_ = len(data_)
        if (len_ != 33):
            # 如果读到全 0
            l_ = list(data_)
            l_.extend([0]*(33 - len_))
            return self._NodeFromBytes(bytes(l_))
        return self._NodeFromBytes(data_)
    
    def _NodeFromBytes(self, data:bytes) -> IndexNode:
        '''
        设置最新位置的索引
        :param index:int 要设置的最新位置的索引
        '''
        return IndexNode(
            prev=int.from_bytes(data[0:8], byteorder='big', signed=False),
            next=int.from_bytes(data[8:16], byteorder='big', signed=False),
            current=int.from_bytes(data[16:24], byteorder='big', signed=False),
            time_stamp_ns=int.from_bytes(data[24:32], byteorder='big', signed=False),
            valid=int.from_bytes(data[32:33], byteorder='big', signed=False))



import random
from BlfIoObjs import *
from BlfIo import BlfIo
import time


if __name__ == '__main__':
    blfio_ = BlfIo()
    # res_ = blfio_.OpenReplayFile('dae.blf')
    # print('OpenReplayFile ', res_)
    # res_ = 0
    # counter_ = 0
    # while (res_ == 0):
    #     res_, obj_ = blfio_.ReplayNext()
    #     counter_+=1
    #     # print('ReplayNext ', res_)
    #     # print(obj_.ToDict())
    # print('counter: ', counter_)
    # res_ = blfio_.CloseReplyFile()
    # print('CloseReplyFile ', res_)


    # res_ = blfio_.OpenLogFile('Z:/blf_io_log/blf_out_{0}.blf'.format(time.strftime('%M%S%H')))
    # print('OpenLogFile ', res_)
    # time_start_ = time.time()
    # for i_ in range(1000):
    #     data_ = [random.randint(0, 0xFF) for i_ in range(8)]
    #     # data_ = []
    #     blfio_.LogNext(CanFdMessage64Py(
    #         channel=3, 
    #         dlc=8, 
    #         id=0xFA, 
    #         data=data_, 
    #         dir=2))
    # res_ = blfio_.CloseLogFile()
    # print(time.time() - time_start_)
    # print('CloseLogFile ', res_)

    # 测试打开不存在的文件读取
    # blfio_.OpenReplayFile('dae.blf')
    # for i_ in range(100):
    #     res_, time_stamp_, time_stamp_type_, obj_ = blfio_.ReplayNext()
    #     print('--------------------------------------')
    #     print('res_ {0}'.format(res_))
    #     print('time_stamp_ {0}'.format(time_stamp_))
    #     print('time_stamp_type_ {0}'.format(time_stamp_type_))
    #     print('obj_ {0}'.format(obj_.ToDict()))

    # 测试写入文件
    res_ = blfio_.OpenLogFile('Z:/blf_io_log/blf_out_{0}.blf'.format(time.strftime('%M%S%H')))
    print('OpenLogFile ', res_)
    time_start_ = time.time()

    log_times_ = 10
    data_ = [0, 0xFF, 0x1F, 0x02, 0x56, 0xAC, 0xDE, 0x06]
    for j_ in range(10000):
        for i_ in range(log_times_):
            blfio_.LogNext(CanMessagePy(
                channel=i_,
                flags=0x01,
                dlc=8,
                id=i_,
                data=data_,
            ))
        for i_ in range(log_times_):
            blfio_.LogNext(CanMessage2Py(
                channel=i_,
                flags=0x00,
                dlc=8,
                id=i_,
                data=data_,
            ))
        for i_ in range(log_times_):
            blfio_.LogNext(CanFdMessagePy(
                channel=i_,
                flags=0x01,
                dlc=8,
                id=i_,
                data=data_,
            ))
        for i_ in range(log_times_):
            blfio_.LogNext(CanFdMessage64Py(
                channel=i_, 
                dlc=8, 
                id=i_, 
                data=data_
            ))
        for i_ in range(log_times_):
            blfio_.LogNext(LinMessagePy(
                channel=i_,
                id=i_,
                dlc=8,
                data=data_,
            ))
        for i_ in range(log_times_):
            blfio_.LogNext(LinMessage2Py(
                data=data_
            ))
        for i_ in range(log_times_):
            blfio_.LogNext(EthernetFramePy(
                sourceAddress=[0x02, 0x80, 0x5E, 0x1F, 0x01, 0x02],
                channel=i_,
                destinationAddress=[0x01, 0xFF, i_, 0x02, 0x34, 0x33],
                dir=0x01,
                type=i_,
                tpid=i_,
                tci=i_,
                payLoadLength=8,
                payLoad=data_,
            ))

    res_ = blfio_.CloseLogFile()
    print(time.time() - time_start_)
    print('CloseLogFile ', res_)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: CanStackNode.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x43\x61nStackNode.proto\x12\x0c\x43\x61nStackNode\x1a\x0c\x43ommon.proto\"}\n\x12\x63\x61n_channel_config\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x05\x12\x0f\n\x07\x62itrate\x18\x02 \x01(\x05\x12\r\n\x05is_fd\x18\x03 \x01(\x08\x12\x12\n\nfd_bitrate\x18\x04 \x01(\x05\x12\x10\n\x08\x62us_type\x18\x05 \x01(\t\x12\x10\n\x08\x61pp_name\x18\x06 \x01(\t\"H\n\x13\x63\x61n_channel_configs\x12\x31\n\x07\x63onfigs\x18\x01 \x03(\x0b\x32 .CanStackNode.can_channel_config\"-\n\x0esubscribe_info\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x05\x12\n\n\x02id\x18\x02 \x01(\x05\"c\n\x11log_start_request\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfile_path\x18\x02 \x01(\t\x12\x1b\n\x13max_log_time_minute\x18\x03 \x01(\r\x12\x10\n\x08\x63hannels\x18\x04 \x03(\x05\" \n\x10log_stop_request\x12\x0c\n\x04name\x18\x01 \x01(\t\"z\n\rcrc_rc_config\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\x05\x12\x16\n\x0e\x61rbitration_id\x18\x02 \x01(\x05\x12\x16\n\x0e\x63rc_bit_starts\x18\x03 \x03(\x05\x12\x15\n\rrc_bit_starts\x18\x05 \x03(\x05\x12\x11\n\tcrc_table\x18\x07 \x03(\x05\"!\n\x05timer\x12\x18\n\x10timer_cycle_time\x18\x01 \x01(\x04\x32\x9f\x05\n\x0c\x43\x61nStackNode\x12.\n\nGetVersion\x12\r.Common.empty\x1a\x0f.Common.version\"\x00\x12\x41\n\nSetConfigs\x12!.CanStackNode.can_channel_configs\x1a\x0e.Common.result\"\x00\x12\x30\n\rStartCanStack\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12/\n\x0cStopCanStack\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12,\n\tClearSend\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12\x31\n\x0e\x43learSubscribe\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12=\n\x08StartLog\x12\x1f.CanStackNode.log_start_request\x1a\x0e.Common.result\"\x00\x12;\n\x07StopLog\x12\x1e.CanStackNode.log_stop_request\x1a\x0e.Common.result\"\x00\x12.\n\x0b\x43learLogger\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x12?\n\x0eSetCrcRcConfig\x12\x1b.CanStackNode.crc_rc_config\x1a\x0e.Common.result\"\x00\x12\x38\n\x0f\x43reatTimerEvent\x12\x13.CanStackNode.timer\x1a\x0e.Common.result\"\x00\x12\x31\n\x0eGetStackStatus\x12\r.Common.empty\x1a\x0e.Common.result\"\x00\x62\x06proto3')



_CAN_CHANNEL_CONFIG = DESCRIPTOR.message_types_by_name['can_channel_config']
_CAN_CHANNEL_CONFIGS = DESCRIPTOR.message_types_by_name['can_channel_configs']
_SUBSCRIBE_INFO = DESCRIPTOR.message_types_by_name['subscribe_info']
_LOG_START_REQUEST = DESCRIPTOR.message_types_by_name['log_start_request']
_LOG_STOP_REQUEST = DESCRIPTOR.message_types_by_name['log_stop_request']
_CRC_RC_CONFIG = DESCRIPTOR.message_types_by_name['crc_rc_config']
_TIMER = DESCRIPTOR.message_types_by_name['timer']
can_channel_config = _reflection.GeneratedProtocolMessageType('can_channel_config', (_message.Message,), {
  'DESCRIPTOR' : _CAN_CHANNEL_CONFIG,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.can_channel_config)
  })
_sym_db.RegisterMessage(can_channel_config)

can_channel_configs = _reflection.GeneratedProtocolMessageType('can_channel_configs', (_message.Message,), {
  'DESCRIPTOR' : _CAN_CHANNEL_CONFIGS,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.can_channel_configs)
  })
_sym_db.RegisterMessage(can_channel_configs)

subscribe_info = _reflection.GeneratedProtocolMessageType('subscribe_info', (_message.Message,), {
  'DESCRIPTOR' : _SUBSCRIBE_INFO,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.subscribe_info)
  })
_sym_db.RegisterMessage(subscribe_info)

log_start_request = _reflection.GeneratedProtocolMessageType('log_start_request', (_message.Message,), {
  'DESCRIPTOR' : _LOG_START_REQUEST,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.log_start_request)
  })
_sym_db.RegisterMessage(log_start_request)

log_stop_request = _reflection.GeneratedProtocolMessageType('log_stop_request', (_message.Message,), {
  'DESCRIPTOR' : _LOG_STOP_REQUEST,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.log_stop_request)
  })
_sym_db.RegisterMessage(log_stop_request)

crc_rc_config = _reflection.GeneratedProtocolMessageType('crc_rc_config', (_message.Message,), {
  'DESCRIPTOR' : _CRC_RC_CONFIG,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.crc_rc_config)
  })
_sym_db.RegisterMessage(crc_rc_config)

timer = _reflection.GeneratedProtocolMessageType('timer', (_message.Message,), {
  'DESCRIPTOR' : _TIMER,
  '__module__' : 'CanStackNode_pb2'
  # @@protoc_insertion_point(class_scope:CanStackNode.timer)
  })
_sym_db.RegisterMessage(timer)

_CANSTACKNODE = DESCRIPTOR.services_by_name['CanStackNode']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CAN_CHANNEL_CONFIG._serialized_start=50
  _CAN_CHANNEL_CONFIG._serialized_end=175
  _CAN_CHANNEL_CONFIGS._serialized_start=177
  _CAN_CHANNEL_CONFIGS._serialized_end=249
  _SUBSCRIBE_INFO._serialized_start=251
  _SUBSCRIBE_INFO._serialized_end=296
  _LOG_START_REQUEST._serialized_start=298
  _LOG_START_REQUEST._serialized_end=397
  _LOG_STOP_REQUEST._serialized_start=399
  _LOG_STOP_REQUEST._serialized_end=431
  _CRC_RC_CONFIG._serialized_start=433
  _CRC_RC_CONFIG._serialized_end=555
  _TIMER._serialized_start=557
  _TIMER._serialized_end=590
  _CANSTACKNODE._serialized_start=593
  _CANSTACKNODE._serialized_end=1264
# @@protoc_insertion_point(module_scope)
